@@ -36,8 +36,8 @@ class RDKIT_CIPLABELER_EXPORT MaxIterationsExceeded
     : public std::runtime_error {
  public:
   explicit MaxIterationsExceeded()
-      : std::runtime_error(
-            "Max Iterations Exceeded in CIP label calculation"){};
+      : std::runtime_error("Max Iterations Exceeded in CIP label calculation") {
+        };
 };
 
 /**
