@@ -19,6 +19,7 @@
 #include <GraphMol/Atom.h>
 #include <GraphMol/AtomIterators.h>
 #include <GraphMol/RingInfo.h>
+#include <GraphMol/Atropisomers.h>
 
 #include <GraphMol/Conformer.h>
 #include <RDGeneral/types.h>
@@ -31,10 +32,12 @@
 #include <GraphMol/MolOps.h>
 #include <GraphMol/ForceFieldHelpers/CrystalFF/TorsionPreferences.h>
 #include <GraphMol/Substruct/SubstructMatch.h>
+#include <GraphMol/MolAlign/AlignMolecules.h>
 #include <boost/dynamic_bitset.hpp>
 #include <iomanip>
 #include <RDGeneral/RDThreads.h>
 #include <typeinfo>
+#include <vector>
 
 #ifdef RDK_BUILD_THREADSAFE_SSS
 #include <future>
@@ -50,15 +53,17 @@
 namespace {
 constexpr double M_PI_2 = 1.57079632679489661923;
 constexpr double ERROR_TOL = 0.00001;
-constexpr size_t MAX_TRACKED_FAILURES = 10;
 // these tolerances, all to detect and filter out bogus conformations, are a
 // delicate balance between sensitive enough to detect obviously bad
 // conformations but not so sensitive that a bunch of ok conformations get
 // filtered out, which slows down the whole conformation generation process
 constexpr double MAX_MINIMIZED_E_PER_ATOM = 0.05;
-constexpr double MAX_MINIMIZED_E_CONTRIB = 0.20;
 constexpr double MIN_TETRAHEDRAL_CHIRAL_VOL = 0.50;
 constexpr double TETRAHEDRAL_CENTERINVOLUME_TOL = 0.30;
+inline bool haveOppositeSign(double a, double b) {
+  return std::signbit(a) ^ std::signbit(b);
+}
+
 }  // namespace
 
 #ifdef RDK_BUILD_THREADSAFE_SSS
@@ -307,12 +312,21 @@ bool _volumeTest(const DistGeom::ChiralSetPtr &chiralSet,
   RDGeom::Point3D v4 = p0 - p4;
   v4.normalize();
 
+  // be more tolerant of tethrahedral centers that are involved in multiple
+  // small rings
+  double volScale = 1;
+  if (chiralSet->d_structureFlags &
+      static_cast<std::uint64_t>(
+          DistGeom::ChiralSetStructureFlags::IN_FUSED_SMALL_RINGS)) {
+    volScale = 0.25;
+  }
+
   RDGeom::Point3D crossp = v1.crossProduct(v2);
   double vol = crossp.dotProduct(v3);
   if (verbose) {
     std::cerr << "   " << fabs(vol) << std::endl;
   }
-  if (fabs(vol) < MIN_TETRAHEDRAL_CHIRAL_VOL) {
+  if (fabs(vol) < volScale * MIN_TETRAHEDRAL_CHIRAL_VOL) {
     return false;
   }
   crossp = v1.crossProduct(v2);
@@ -320,7 +334,7 @@ bool _volumeTest(const DistGeom::ChiralSetPtr &chiralSet,
   if (verbose) {
     std::cerr << "   " << fabs(vol) << std::endl;
   }
-  if (fabs(vol) < MIN_TETRAHEDRAL_CHIRAL_VOL) {
+  if (fabs(vol) < volScale * MIN_TETRAHEDRAL_CHIRAL_VOL) {
     return false;
   }
   crossp = v1.crossProduct(v3);
@@ -328,7 +342,7 @@ bool _volumeTest(const DistGeom::ChiralSetPtr &chiralSet,
   if (verbose) {
     std::cerr << "   " << fabs(vol) << std::endl;
   }
-  if (fabs(vol) < MIN_TETRAHEDRAL_CHIRAL_VOL) {
+  if (fabs(vol) < volScale * MIN_TETRAHEDRAL_CHIRAL_VOL) {
     return false;
   }
   crossp = v2.crossProduct(v3);
@@ -336,7 +350,7 @@ bool _volumeTest(const DistGeom::ChiralSetPtr &chiralSet,
   if (verbose) {
     std::cerr << "   " << fabs(vol) << std::endl;
   }
-  return fabs(vol) >= MIN_TETRAHEDRAL_CHIRAL_VOL;
+  return fabs(vol) >= volScale * MIN_TETRAHEDRAL_CHIRAL_VOL;
 }
 
 bool _sameSide(const RDGeom::Point3D &v1, const RDGeom::Point3D &v2,
@@ -492,16 +506,10 @@ bool firstMinimization(RDGeom::PointPtrVect *positions,
             << std::endl;
 #endif
 
-  // check that neither the energy nor any of the contributions to it are
-  // too high (this is part of github #971)
-  if (local_e / positions->size() >= MAX_MINIMIZED_E_PER_ATOM ||
-      (e_contribs.size() &&
-       *(std::max_element(e_contribs.begin(), e_contribs.end())) >
-           MAX_MINIMIZED_E_CONTRIB)) {
+  // check that the energy is not too high (this is part of github #971)
+  if (local_e / positions->size() >= MAX_MINIMIZED_E_PER_ATOM) {
 #ifdef DEBUG_EMBEDDING
-    std::cerr << " Energy fail: " << local_e / positions->size() << " "
-              << *(std::max_element(e_contribs.begin(), e_contribs.end()))
-              << std::endl;
+    std::cerr << " Energy fail: " << local_e / positions->size() << std::endl;
 #endif
     gotCoords = false;
   }
@@ -543,8 +551,8 @@ bool checkChiralCenters(const RDGeom::PointPtrVect *positions,
         chiralSet->d_idx4, *positions);
     double lb = chiralSet->getLowerVolumeBound();
     double ub = chiralSet->getUpperVolumeBound();
-    if ((lb > 0 && vol < lb && (lb - vol) / lb > .2) ||
-        (ub < 0 && vol > ub && (vol - ub) / ub > .2)) {
+    if ((lb > 0 && vol < lb && (vol / lb < .8 || haveOppositeSign(vol, lb))) ||
+        (ub < 0 && vol > ub && (vol / ub < .8 || haveOppositeSign(vol, ub)))) {
 #ifdef DEBUG_EMBEDDING
       std::cerr << " fail! (" << chiralSet->d_idx0 << ") iter: "
                 << " " << vol << " " << lb << "-" << ub << std::endl;
@@ -572,7 +580,7 @@ bool minimizeFourthDimension(RDGeom::PointPtrVect *positions,
   }
 
   field2->initialize();
-  // std::cerr<<"FIELD2 E: "<<field2->calcEnergy()<<std::endl;
+  // std::cerr << "FIELD2 E: " << field2->calcEnergy() << std::endl;
   if (field2->calcEnergy() > ERROR_TOL) {
     int needMore = 1;
     while (needMore) {
@@ -629,9 +637,8 @@ bool minimizeWithExpTorsions(RDGeom::PointPtrVect &positions,
   if (embedParams.useBasicKnowledge) {
     // create a force field with only the impropers
     std::unique_ptr<ForceFields::ForceField> field2(
-        DistGeom::construct3DImproperForceField(
-            *eargs.mmat, positions3D, eargs.etkdgDetails->improperAtoms,
-            eargs.etkdgDetails->atomNums));
+        DistGeom::construct3DImproperForceField(*eargs.mmat, positions3D,
+                                                *eargs.etkdgDetails));
     if (embedParams.useRandomCoords && embedParams.coordMap != nullptr) {
       for (const auto &v : *embedParams.coordMap) {
         field2->fixedPoints().push_back(v.first);
@@ -733,6 +740,17 @@ bool doubleBondStereoChecks(const RDGeom::PointPtrVect &positions,
 bool finalChiralChecks(RDGeom::PointPtrVect *positions,
                        const detail::EmbedArgs &eargs,
                        EmbedParameters &embedParams) {
+  // confirm chiral volumes
+  if (!checkChiralCenters(positions, eargs, embedParams)) {
+    if (embedParams.trackFailures) {
+#ifdef RDK_BUILD_THREADSAFE_SSS
+      std::lock_guard<std::mutex> lock(GetFailMutex());
+#endif
+      embedParams.failures[EmbedFailureCauses::CHECK_CHIRAL_CENTERS2]++;
+    }
+    return false;
+  }
+
   // "distance matrix" chirality test
   std::set<int> atoms;
   for (const auto &chiralSet : *eargs.chiralCenters) {
@@ -780,6 +798,8 @@ bool finalChiralChecks(RDGeom::PointPtrVect *positions,
       return false;
     }
   }
+  // FIX: do we need some kind of sanity check here for the non-atomic
+  // situations (e.g. atropisomers)?
 
   return true;
 }
@@ -1021,19 +1041,33 @@ void findChiralSets(const ROMol &mol, DistGeom::VECT_CHIRALSET &chiralCenters,
           nbrs.insert(nbrs.end(), atom->getIdx());
         }
 
+        // set a flag for tetrahedral centers that are in multiple small rings
+        auto numSmallRings = 0u;
+        constexpr int smallRingSize = 5;
+        for (const auto sz : mol.getRingInfo()->atomRingSizes(atom->getIdx())) {
+          if (sz < smallRingSize) {
+            ++numSmallRings;
+          }
+        }
+        std::uint64_t structureFlags = 0;
+        if (numSmallRings > 1) {
+          structureFlags = static_cast<std::uint64_t>(
+              DistGeom::ChiralSetStructureFlags::IN_FUSED_SMALL_RINGS);
+        }
+
         // now create a chiral set and set the upper and lower bound on the
         // volume
         if (chiralType == Atom::CHI_TETRAHEDRAL_CCW) {
           // positive chiral volume
-          auto *cset =
-              new DistGeom::ChiralSet(atom->getIdx(), nbrs[0], nbrs[1], nbrs[2],
-                                      nbrs[3], volLowerBound, volUpperBound);
+          auto *cset = new DistGeom::ChiralSet(atom->getIdx(), nbrs[0], nbrs[1],
+                                               nbrs[2], nbrs[3], volLowerBound,
+                                               volUpperBound, structureFlags);
           DistGeom::ChiralSetPtr cptr(cset);
           chiralCenters.push_back(cptr);
         } else if (chiralType == Atom::CHI_TETRAHEDRAL_CW) {
-          auto *cset =
-              new DistGeom::ChiralSet(atom->getIdx(), nbrs[0], nbrs[1], nbrs[2],
-                                      nbrs[3], -volUpperBound, -volLowerBound);
+          auto *cset = new DistGeom::ChiralSet(atom->getIdx(), nbrs[0], nbrs[1],
+                                               nbrs[2], nbrs[3], -volUpperBound,
+                                               -volLowerBound, structureFlags);
           DistGeom::ChiralSetPtr cptr(cset);
           chiralCenters.push_back(cptr);
         } else {
@@ -1045,16 +1079,73 @@ void findChiralSets(const ROMol &mol, DistGeom::VECT_CHIRALSET &chiralCenters,
             // the coordMap
             // there's no sense doing 3-rings because those are a nightmare
           } else {
-            auto *cset = new DistGeom::ChiralSet(
-                atom->getIdx(), nbrs[0], nbrs[1], nbrs[2], nbrs[3], 0.0, 0.0);
+            auto *cset = new DistGeom::ChiralSet(atom->getIdx(), nbrs[0],
+                                                 nbrs[1], nbrs[2], nbrs[3], 0.0,
+                                                 0.0, structureFlags);
             DistGeom::ChiralSetPtr cptr(cset);
             tetrahedralCenters.push_back(cptr);
           }
         }
       }  // if block -chirality check
-    }    // if block - heavy atom check
-  }      // for loop over atoms
-}  // end of _findChiralSets
+    }  // if block - heavy atom check
+  }  // for loop over atoms
+
+  // now do atropisomers
+  for (const auto &bond : mol.bonds()) {
+    if (bond->getStereo() != Bond::BondStereo::STEREOATROPCCW &&
+        bond->getStereo() != Bond::BondStereo::STEREOATROPCW) {
+      continue;
+    }
+    Atropisomers::AtropAtomAndBondVec atomsAndBonds[2];
+    Atropisomers::getAtropisomerAtomsAndBonds(bond, atomsAndBonds, mol);
+    // make a chiral set for the atropisomeric bond
+    // we start with only managing cases where there are two exo-substituents on
+    // at least one side
+    if (atomsAndBonds[0].second.size() != 2 &&
+        atomsAndBonds[1].second.size() != 2) {
+      BOOST_LOG(rdWarningLog)
+          << "Atropisomer bond stereochemistry not used for bond "
+          << bond->getIdx()
+          << ", which does not have two exo substituents on at least one side."
+          << std::endl;
+      continue;
+    }
+    int idx0 = atomsAndBonds[0].first->getIdx();
+    int idx1 = atomsAndBonds[1].first->getIdx();
+
+    int nbr1 = atomsAndBonds[0].second[0]->getOtherAtomIdx(idx0);
+    int nbr2 = 0;
+    int nbr3 = 0;
+    int nbr4 = 0;
+    if (atomsAndBonds[0].second.size() == 2) {
+      nbr2 = atomsAndBonds[0].second[1]->getOtherAtomIdx(idx0);
+      nbr3 = atomsAndBonds[1].second[0]->getOtherAtomIdx(idx1);
+      if (atomsAndBonds[1].second.size() == 2) {
+        nbr4 = atomsAndBonds[1].second[1]->getOtherAtomIdx(idx1);
+      } else {
+        nbr4 = idx0;
+      }
+    } else {
+      nbr2 = atomsAndBonds[1].second[0]->getOtherAtomIdx(idx1);
+      nbr3 = atomsAndBonds[1].second[1]->getOtherAtomIdx(idx1);
+      nbr4 = idx0;
+    }
+    INT_VECT nbrs = {nbr1, nbr2, nbr3, nbr4};
+
+    // FIX: these numbers are empirical and should be revisited
+    double volLowerBound = 1.0;
+    double volUpperBound = 100.0;
+    if (bond->getStereo() == Bond::BondStereo::STEREOATROPCCW) {
+      std::swap(volLowerBound, volUpperBound);
+      volLowerBound *= -1;
+      volUpperBound *= -1;
+    }
+    auto *cset = new DistGeom::ChiralSet(idx0, nbrs[0], nbrs[1], nbrs[2],
+                                         nbrs[3], volLowerBound, volUpperBound);
+    DistGeom::ChiralSetPtr cptr(cset);
+    chiralCenters.push_back(cptr);
+  }
+}
 
 void adjustBoundsMatFromCoordMap(
     DistGeom::BoundsMatPtr mmat, unsigned int,
@@ -1164,7 +1255,7 @@ bool _isConfFarFromRest(
   RDGeom::Point3DConstPtrVect prbPoints(selfMatches[0].size());
   _fillAtomPositions(refPoints, conf, mol, selfMatches[0]);
 
-  double ssrThres = conf.getNumAtoms() * threshold * threshold;
+  double ssrThres = selfMatches[0].size() * threshold * threshold;
   for (const auto &match : selfMatches) {
     for (auto confi = mol.beginConformers(); confi != mol.endConformers();
          ++confi) {
@@ -1287,16 +1378,24 @@ std::vector<std::vector<unsigned int>> getMolSelfMatches(
     MolOps::RemoveHsParameters ps;
     bool sanitize = false;
     MolOps::removeHs(tmol, ps, sanitize);
+
+    std::unique_ptr<RWMol> prbMolSymm;
+    if (params.symmetrizeConjugatedTerminalGroupsForPruning) {
+      prbMolSymm.reset(new RWMol(tmol));
+      MolAlign::details::symmetrizeTerminalAtoms(*prbMolSymm);
+    }
+    const auto &prbMolForMatch = prbMolSymm ? *prbMolSymm : tmol;
+
     SubstructMatchParameters sssps;
     sssps.maxMatches = 1;
     // provides the atom indices in the molecule corresponding
     // to the indices in the H-stripped version
-    auto strippedMatch = SubstructMatch(mol, tmol, sssps);
+    auto strippedMatch = SubstructMatch(mol, prbMolForMatch, sssps);
     CHECK_INVARIANT(strippedMatch.size() == 1, "expected match not found");
 
     sssps.maxMatches = 1000;
     sssps.uniquify = false;
-    auto heavyAtomMatches = SubstructMatch(tmol, tmol, sssps);
+    auto heavyAtomMatches = SubstructMatch(tmol, prbMolForMatch, sssps);
     for (const auto &match : heavyAtomMatches) {
       res.emplace_back(0);
       res.back().reserve(match.size());
@@ -1329,7 +1428,7 @@ void EmbedMultipleConfs(ROMol &mol, INT_VECT &res, unsigned int numConfs,
 #ifdef RDK_BUILD_THREADSAFE_SSS
     std::lock_guard<std::mutex> lock(GetFailMutex());
 #endif
-    params.failures.resize(MAX_TRACKED_FAILURES);
+    params.failures.resize(EmbedFailureCauses::END_OF_ENUM);
     std::fill(params.failures.begin(), params.failures.end(), 0);
   }
   if (!mol.getNumAtoms()) {
@@ -1378,6 +1477,12 @@ void EmbedMultipleConfs(ROMol &mol, INT_VECT &res, unsigned int numConfs,
         << std::endl;
     coordMap = nullptr;
   }
+  boost::dynamic_bitset<> constrainedAtoms(mol.getNumAtoms());
+  if (coordMap) {
+    for (const auto &entry : *coordMap) {
+      constrainedAtoms.set(entry.first);
+    }
+  }
 
   if (molFrags.size() > 1 && params.boundsMat != nullptr) {
     BOOST_LOG(rdWarningLog)
@@ -1396,6 +1501,7 @@ void EmbedMultipleConfs(ROMol &mol, INT_VECT &res, unsigned int numConfs,
     unsigned int nAtoms = piece->getNumAtoms();
 
     ForceFields::CrystalFF::CrystalFFDetails etkdgDetails;
+    etkdgDetails.constrainedAtoms = constrainedAtoms;
     EmbeddingOps::initETKDG(piece.get(), params, etkdgDetails);
 
     DistGeom::BoundsMatPtr mmat;
@@ -1469,7 +1575,11 @@ void EmbedMultipleConfs(ROMol &mol, INT_VECT &res, unsigned int numConfs,
     }
 #endif
   }
-  auto selfMatches = detail::getMolSelfMatches(mol, params);
+  std::vector<std::vector<unsigned int>> selfMatches;
+  if (params.pruneRmsThresh > 0.0) {
+    selfMatches = detail::getMolSelfMatches(mol, params);
+  }
+
   for (unsigned int ci = 0; ci < confs.size(); ++ci) {
     auto &conf = confs[ci];
     if (confsOk[ci]) {
