@@ -67,15 +67,6 @@ void convertToBonds(const VECT_INT_VECT &res, VECT_INT_VECT &brings,
 
 }  // end of namespace RingUtils
 
-namespace RDKit {
-namespace {
-bool bondIsDative(const RDKit::Bond &bond) {
-  auto bt = bond.getBondType();
-  return bt == Bond::BondType::DATIVE || bt == Bond::BondType::DATIVEL ||
-         bt == Bond::BondType::DATIVER || bt == Bond::BondType::DATIVEONE;
-}
-}  // namespace
-}  // namespace RDKit
 namespace FindRings {
 using namespace RDKit;
 
@@ -204,8 +195,8 @@ void findSSSRforDupCands(const ROMol &mol, VECT_INT_VECT &res,
           }
         }
       }  // end of loop over new rings found
-    }    // end if (dupCand.size() > 1)
-  }      // end of loop over all set of duplicate candidates
+    }  // end if (dupCand.size() > 1)
+  }  // end of loop over all set of duplicate candidates
 }
 
 auto compRingSize = [](const auto &v1, const auto &v2) {
@@ -573,7 +564,7 @@ void findRingsD3Node(const ROMol &tMol, VECT_INT_VECT &res,
         }
       }
     }  // doing node of degree 3 - end of found only 1 smallest ring
-  }    // end of found less than 3 smallest ring for the degree 3 node
+  }  // end of found less than 3 smallest ring for the degree 3 node
 }
 
 int greatestComFac(long curfac, long nfac) {
@@ -753,7 +744,7 @@ int BFSWorkspace::smallestRingsBfs(const ROMol &mol, int root,
         }
       }
     }  // end of loop over neighbors of current atom
-  }    // moving to the next node
+  }  // moving to the next node
 
   // if we are here we should have found everything around the node
   return rdcast<unsigned int>(rings.size());
@@ -870,7 +861,7 @@ int findSSSR(const ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
   if (mol.getRingInfo()->isInitialized()) {
     mol.getRingInfo()->reset();
   }
-  mol.getRingInfo()->initialize();
+  mol.getRingInfo()->initialize(FIND_RING_TYPE_SSSR);
   RINGINVAR_SET invars;
 
   unsigned int nats = mol.getNumAtoms();
@@ -887,7 +878,7 @@ int findSSSR(const ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
   while (firstB != lastB) {
     const Bond *bond = mol[*firstB];
     if (bond->getBondType() == Bond::ZERO ||
-        (!includeDativeBonds && bondIsDative(*bond))) {
+        (!includeDativeBonds && isDative(*bond))) {
       activeBonds[bond->getIdx()] = 0;
     }
     ++firstB;
@@ -905,7 +896,7 @@ int findSSSR(const ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
     atomDegreesWithZeroOrderBonds[i] = deg;
     for (const auto bond : mol.atomBonds(atom)) {
       if (bond->getBondType() == Bond::ZERO ||
-          (!includeDativeBonds && bondIsDative(*bond))) {
+          (!includeDativeBonds && isDative(*bond))) {
         atomDegrees[i]--;
       }
     }
@@ -1004,8 +995,8 @@ int findSSSR(const ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
       }  // end of degree two nodes
       else if (nAtomsDone <
                curFrag.size()) {  // now deal with higher degree nodes
-        // this is brutal - we have no degree 2 nodes - find the first possible
-        // degree 3 node
+        // this is brutal - we have no degree 2 nodes - find the first
+        // possible degree 3 node
         int cand = -1;
         for (INT_VECT_CI aidi = curFrag.begin(); aidi != curFrag.end();
              aidi++) {
@@ -1027,7 +1018,7 @@ int findSSSR(const ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
         ++nAtomsDone;
         FindRings::trimBonds(cand, mol, changed, atomDegrees, activeBonds);
       }  // done with degree 3 node
-    }    // done finding rings in this fragment
+    }  // done finding rings in this fragment
 
 #if 0
         std::cerr<<"\n\nFOUND:\n";
@@ -1138,6 +1129,9 @@ int symmetrizeSSSR(ROMol &mol, VECT_INT_VECT &res, bool includeDativeBonds) {
   // avoid repeating this work
   findSSSR(mol, sssrs, includeDativeBonds);
 
+  // reinit as SYMM_SSSR
+  mol.getRingInfo()->initialize(FIND_RING_TYPE_SYMM_SSSR);
+
   res.reserve(sssrs.size());
   for (const auto &r : sssrs) {
     res.emplace_back(r);
@@ -1269,7 +1263,7 @@ void fastFindRings(const ROMol &mol) {
     mol.getRingInfo()->reset();
   }
 
-  mol.getRingInfo()->initialize();
+  mol.getRingInfo()->initialize(FIND_RING_TYPE_FAST);
 
   VECT_INT_VECT res;
   res.resize(0);
