@@ -21,12 +21,12 @@
 #include <GraphMol/RWMol.h>
 #include <GraphMol/MolOps.h>
 #include <GraphMol/Depictor/RDDepictor.h>
-#include <GraphMol/FileParsers/MolFileStereochem.h>
 
 #include <RDGeneral/BoostStartInclude.h>
 #include <boost/lexical_cast.hpp>
 #include <boost/property_tree/ptree.hpp>
 #include <boost/property_tree/json_parser.hpp>
+#include <boost/algorithm/string.hpp>
 #include <RDGeneral/BoostEndInclude.h>
 #include <limits>
 #include <cmath>
@@ -76,7 +76,7 @@ void prepareMolForDrawing(RWMol &mol, bool kekulize, bool addChiralHs,
     RDDepict::compute2DCoords(mol, nullptr, canonOrient);
   }
   if (wedgeBonds) {
-    WedgeMolBonds(mol, &mol.getConformer());
+    Chirality::wedgeMolBonds(mol, &mol.getConformer());
   }
   if (wavyBonds) {
     addWavyBondsForStereoAny(mol);
@@ -130,40 +130,64 @@ void get_rgba(const boost::property_tree::ptree &node, DrawColour &colour) {
   }
 }
 
-void get_colour_option(boost::property_tree::ptree *pt, const char *pnm,
+void get_colour_option(const boost::property_tree::ptree &pt, const char *pnm,
                        DrawColour &colour) {
   PRECONDITION(pnm && strlen(pnm), "bad property name");
-  if (pt->find(pnm) == pt->not_found()) {
+  if (pt.find(pnm) == pt.not_found()) {
     return;
   }
 
-  const auto &node = pt->get_child(pnm);
+  const auto &node = pt.get_child(pnm);
   get_rgba(node, colour);
 }
 
-void get_colour_palette_option(boost::property_tree::ptree *pt, const char *pnm,
-                               ColourPalette &palette) {
+void get_colour_palette_option(const boost::property_tree::ptree &pt,
+                               const char *pnm, ColourPalette &palette) {
   PRECONDITION(pnm && strlen(pnm), "bad property name");
-  if (pt->find(pnm) == pt->not_found()) {
+  static const std::map<std::string, void (*)(ColourPalette &)>
+      PRESET_PALETTE_MAP{
+          {"default", assignDefaultPalette},
+          {"avalon", assignAvalonPalette},
+          {"cdk", assignCDKPalette},
+          {"darkmode", assignDarkModePalette},
+          {"bw", assignBWPalette},
+      };
+  auto atomColourPaletteIt = pt.find(pnm);
+  if (atomColourPaletteIt == pt.not_found()) {
     return;
   }
-
-  for (const auto &atomicNumNodeIt : pt->get_child(pnm)) {
-    int atomicNum = boost::lexical_cast<int>(atomicNumNodeIt.first);
-    DrawColour colour;
-    get_rgba(atomicNumNodeIt.second, colour);
-    palette[atomicNum] = colour;
+  // Does the "atomColourPalette" key correspond to a terminal value?
+  if (atomColourPaletteIt->second.empty()) {
+    const auto &v = atomColourPaletteIt->second.data();
+    if (v.empty()) {
+      return;
+    }
+    auto paletteName = boost::lexical_cast<std::string>(v);
+    boost::algorithm::to_lower(paletteName);
+    auto preSetPaletteIt = PRESET_PALETTE_MAP.find(paletteName);
+    if (preSetPaletteIt == PRESET_PALETTE_MAP.end()) {
+      return;
+    }
+    // Populate the palette calling the corresponding function
+    preSetPaletteIt->second(palette);
+  } else {
+    for (const auto &atomicNumNode : atomColourPaletteIt->second) {
+      int atomicNum = boost::lexical_cast<int>(atomicNumNode.first);
+      DrawColour colour;
+      get_rgba(atomicNumNode.second, colour);
+      palette[atomicNum] = colour;
+    }
   }
 }
 
-void get_highlight_style_option(boost::property_tree::ptree *pt,
+void get_highlight_style_option(const boost::property_tree::ptree &pt,
                                 const char *pnm,
                                 MultiColourHighlightStyle &mchs) {
   PRECONDITION(pnm && strlen(pnm), "bad property name");
-  if (pt->find(pnm) == pt->not_found()) {
+  if (pt.find(pnm) == pt.not_found()) {
     return;
   }
-  const auto &node = pt->get_child(pnm);
+  const auto &node = pt.get_child(pnm);
   auto styleStr = node.get_value<std::string>();
   if (styleStr == "Lasso") {
     mchs = MultiColourHighlightStyle::LASSO;
@@ -233,22 +257,22 @@ void updateMolDrawOptionsFromJSON(MolDrawOptions &opts,
   PT_OPT_GET(drawMolsSameScale);
   PT_OPT_GET(useComplexQueryAtomSymbols);
 
-  get_colour_option(&pt, "highlightColour", opts.highlightColour);
-  get_colour_option(&pt, "backgroundColour", opts.backgroundColour);
-  get_colour_option(&pt, "queryColour", opts.queryColour);
-  get_colour_option(&pt, "legendColour", opts.legendColour);
-  get_colour_option(&pt, "symbolColour", opts.symbolColour);
-  get_colour_option(&pt, "annotationColour", opts.annotationColour);
-  get_colour_option(&pt, "variableAttachmentColour",
+  get_colour_option(pt, "highlightColour", opts.highlightColour);
+  get_colour_option(pt, "backgroundColour", opts.backgroundColour);
+  get_colour_option(pt, "queryColour", opts.queryColour);
+  get_colour_option(pt, "legendColour", opts.legendColour);
+  get_colour_option(pt, "symbolColour", opts.symbolColour);
+  get_colour_option(pt, "annotationColour", opts.annotationColour);
+  get_colour_option(pt, "variableAttachmentColour",
                     opts.variableAttachmentColour);
-  get_colour_palette_option(&pt, "atomColourPalette", opts.atomColourPalette);
+  get_colour_palette_option(pt, "atomColourPalette", opts.atomColourPalette);
   if (pt.find("atomLabels") != pt.not_found()) {
     for (const auto &item : pt.get_child("atomLabels")) {
       opts.atomLabels[boost::lexical_cast<int>(item.first)] =
           item.second.get_value<std::string>();
     }
   }
-  get_highlight_style_option(&pt, "multiColourHighlightStyle",
+  get_highlight_style_option(pt, "multiColourHighlightStyle",
                              opts.multiColourHighlightStyle);
 }
 
@@ -502,7 +526,8 @@ void drawMolACS1996(MolDraw2D &drawer, const ROMol &mol,
 // ****************************************************************************
 void setACS1996Options(MolDrawOptions &opts, double meanBondLen) {
   if (meanBondLen <= 0.0) {
-    throw ValueErrorException("ACS1996Options requires mean bond length > 0.0.");
+    throw ValueErrorException(
+        "ACS1996Options requires mean bond length > 0.0.");
   }
   opts.bondLineWidth = 0.6;
   opts.scaleBondWidth = false;
