@@ -414,9 +414,9 @@ TEST_CASE("tautomer v2") {
             {{"CC(=O)CC(=O)C", "C=C(O)CC(=O)C", "CC(=O)C=C(O)C",
               "C=C(O)C=C(O)C", "C=C(O)CC(O)=C"},
              {}},
-            {{"CN=CF", "C=NCF"}, {"CNCF"}},
-            {{"CN=C(C)F", "C=NC(C)F", "CNC(=C)F"}, {}},
-            {{"Cc1n[nH]cc1", "Cc1[nH][n]cc1", "CC1N=NCC=1", "CC1=NN=CC1"}, {}},
+            {{"CN=CCF", "CNC=CF"}, {"C=NCCF", "CNCCF"}},
+            {{"CN=C(C)F", "CNC(=C)F"}, {"C=NC(C)F"}},
+            {{"Cc1n[nH]cc1", "Cc1[nH][n]cc1", "CC1=NN=CC1", "CC1N=NCC=1"}, {}},
             {{"O=C1C=CC(=O)C=C1"}, {"Oc1ccc(O)cc1", "O=C1C=CC(O)C=C1"}},
             {{"CC(=O)CCC(=O)C", "CC(=O)CCC(O)=C", "C=C(O)CCC(O)=C"},
              {"CC(O)C=CC(=O)C", "CC(=O)C=CC(O)C"}},
@@ -435,9 +435,10 @@ TEST_CASE("tautomer v2") {
             {{"S=C1N=CN=C2NC=NC12", "S=C1NC=NC2N=CNC1=2"}, {}},
             {{"CC1=CN=CN1", "CC1CN=CN=1"}, {}},
             {{"N1C(=O)NC(=O)C2C=NNC=21", "N1C(=O)NC(=O)C2=CNNC2=1",
-              "N1C(=O)NC(=O)C2CN=NC=21", "N1C(=O)NC(=O)C2=CNNC2=1",
-              "N1C(=O)NC(=O)C2CN=NC2=1"},
-             {}},
+              "N1C(=O)NC(=O)C2=CNNC2=1", "N1C(=O)NC(=O)C2CN=NC2=1"},
+             {
+                 "N1C(=O)NC(=O)C2CN=NC=21",
+             }},
             // ---------------------------
             // more stereochemistry
             // ---------------------------
@@ -446,12 +447,12 @@ TEST_CASE("tautomer v2") {
             {{"C/C=C/O", "C/C=C\\O", "CC=CO"}, {}},
             {{"C/C=C/C=O", "C/C=C\\C=O", "CC=CC=O"}, {}},
             {{"C/C=C/CC=O"}, {"C/C=C\\CC=O", "CC=CCC=O"}},
+            {{"C1C=CC=C2C=C(O)NC=12", "C1C=CC=C2CC(=O)NC=12"}, {}},
             // ---------------------------
             // some areas for potential improvement
             // ---------------------------
             // these two are tautomers, but the current algorithm does not catch
             // them
-            {{"C1C=CC=C2C=C(O)NC=12"}, {"C1C=CC=C2CC(=O)NC=12"}},
             // problematic because pyridine is recognized as tautomeric
             {{"c1ccccc1/C=C/c1ncccc1", "c1ccccc1/C=C\\c1ncccc1",
               "c1ccccc1C=Cc1ncccc1"},
@@ -464,7 +465,7 @@ TEST_CASE("tautomer v2") {
       auto ref =
           MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomTautomerv2);
       for (auto i = 1u; i < same.size(); ++i) {
-        INFO(same[0] + "->" + same[i]);
+        INFO(same[0] + "." + same[i]);
         std::unique_ptr<RWMol> m2{SmilesToMol(same[i])};
         REQUIRE(m2);
         RWMol cp(*m2);
@@ -473,7 +474,7 @@ TEST_CASE("tautomer v2") {
         CHECK(hsh == ref);
       }
       for (auto i = 0u; i < diff.size(); ++i) {
-        INFO(same[0] + "->" + diff[i]);
+        INFO(same[0] + "." + diff[i]);
         std::unique_ptr<RWMol> m2{SmilesToMol(diff[i])};
         REQUIRE(m2);
         RWMol cp(*m2);
@@ -500,7 +501,7 @@ TEST_CASE("tautomer v2") {
          "[CH3]-[C](-[CH3])(-[CH3])-[CH]=[O]_0_0"},
         {"CC(C)=CO", "C[C](C)[CH][O]_1_0", "[CH3]-[C](-[CH3]):[C]:[O]_2_0"},
         {"COC=O", "CO[CH][O]_0_0", "[CH3]-[O]:[C]:[O]_1_0"},
-        {"CNC=O", "C[N][CH][O]_1_0", "[C]:[N]:[C]:[O]_5_0"},
+        {"CNC=O", "C[N][CH][O]_1_0", "[CH3]-[N]:[C]:[O]_2_0"},
         {"CN(C)C=O", "CN(C)[CH][O]_0_0", "[CH3]-[N](-[CH3]):[C]:[O]_1_0"},
         {"CC(C)(C)NC=O", "CC(C)(C)[N][CH][O]_1_0",
          "[CH3]-[C](-[CH3])(-[CH3])-[N]:[C]:[O]_2_0"},
@@ -510,7 +511,7 @@ TEST_CASE("tautomer v2") {
         {"CC=CC(=O)C", "C[CH][CH][C](C)[O]_0_0",
          "[C]:[C](:[O]):[C]:[C]-[CH3]_5_0"},
         {"N1C=CCC(F)C1", "FC1C[CH][CH][N]C1_1_0",
-         "[F]-[CH]1-[C]:[N]:[C]:[C]-[CH2]-1_5_0"},
+         "[F]-[CH]1-[CH2]-[C]:[C]:[N]-[CH2]-1_3_0"},
         {"CCC=C(O)C", "CC[CH][C](C)[O]_1_0",
          "[C]:[C](:[O]):[C]-[CH2]-[CH3]_5_0"},
         {"CCCC(=O)C", "CCC[C](C)[O]_0_0", "[C]:[C](:[O]):[C]-[CH2]-[CH3]_5_0"},
@@ -752,3 +753,125 @@ TEST_CASE("Github Issue #6855 MakeScaffoldGeneric isotope removal") {
     }
   }
 }
+
+TEST_CASE("Github Issue #6472 non-matching element and anononymous graph") {
+  SECTION("Element graph test1") {
+    auto mol = "C1COC(C1)C1=NC=NC=C1"_smiles;
+    REQUIRE(mol);
+    {
+      RWMol cp(*mol);
+      auto hsh = MolHash::MolHash(&cp, MolHash::HashFunction::ElementGraph);
+      CHECK(hsh == "C1COC(C2CCNCN2)C1");
+    }
+  }
+  SECTION("Element graph test2") {
+    auto mol = "C1CC(N=CN1)C1=CC=CO1"_smiles;
+    REQUIRE(mol);
+    {
+      RWMol cp(*mol);
+      auto hsh = MolHash::MolHash(&cp, MolHash::HashFunction::ElementGraph);
+      CHECK(hsh == "C1COC(C2CCNCN2)C1");
+    }
+  }
+  SECTION("Anonymous graph test 1") {
+    auto mol = "C1COC(C1)C1=NC=NC=C1"_smiles;
+    REQUIRE(mol);
+
+    {
+      RWMol cp(*mol);
+      auto hsh = MolHash::MolHash(&cp, MolHash::HashFunction::AnonymousGraph);
+      CHECK(hsh == "*1***(*2****2)**1");
+    }
+  }
+  SECTION("Anonymous graph test 2") {
+    auto mol = "C1CC(N=CN1)C1=CC=CO1"_smiles;
+    REQUIRE(mol);
+
+    {
+      RWMol cp(*mol);
+      auto hsh = MolHash::MolHash(&cp, MolHash::HashFunction::AnonymousGraph);
+      CHECK(hsh == "*1***(*2****2)**1");
+    }
+  }
+}
+
+TEST_CASE("tautomer overreach") {
+  SECTION("as reported") {
+    auto mol = "C1=CN(C[C@H]2CNCCO2)N=C1"_smiles;
+    REQUIRE(mol);
+    {
+      RWMol cp(*mol);
+      auto hsh =
+          MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomTautomerv2);
+      CHECK(
+          hsh ==
+          "[C]1:[C]:[N]:[N](-[CH2]-[C@H]2-[CH2]-[NH]-[CH2]-[CH2]-[O]-2):[C]:1_3_0");
+    }
+  }
+  SECTION("dbw example") {
+    auto mol = "c1cccn1C[C@H](C)COC"_smiles;
+    REQUIRE(mol);
+    {
+      RWMol cp(*mol);
+      auto hsh =
+          MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomTautomerv2);
+      CHECK(
+          hsh ==
+          "[CH3]-[O]-[CH2]-[C@@H](-[CH3])-[CH2]-[n]1:[cH]:[cH]:[cH]:[cH]:1_0_0");
+    }
+  }
+}
+
+TEST_CASE("HetAtomProtomerv2") {
+  SECTION("matches") {
+    // pairs of {molecules with the same hash} {molecules with different hashes
+    // from those}
+    std::vector<std::pair<std::vector<std::string>, std::vector<std::string>>>
+        data = {
+            // example from the NextMove documentation
+            {{"Cc1c[nH]cn1", "Cc1cnc[nH]1", "Cc1c[nH]c[nH+]1"}, {}},
+            {{"CC=CO", "CCC=O"}, {"C=CCO"}},
+
+        };
+    for (const auto &[same, diff] : data) {
+      std::unique_ptr<RWMol> m{SmilesToMol(same[0])};
+      REQUIRE(m);
+      RWMol cp(*m);
+      auto ref =
+          MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomProtomerv2);
+      for (auto i = 1u; i < same.size(); ++i) {
+        INFO(same[0] + "->" + same[i]);
+        std::unique_ptr<RWMol> m2{SmilesToMol(same[i])};
+        REQUIRE(m2);
+        RWMol cp(*m2);
+        auto hsh =
+            MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomProtomerv2);
+        CHECK(hsh == ref);
+      }
+      for (auto i = 0u; i < diff.size(); ++i) {
+        INFO(same[0] + "->" + diff[i]);
+        std::unique_ptr<RWMol> m2{SmilesToMol(diff[i])};
+        REQUIRE(m2);
+        RWMol cp(*m2);
+        auto hsh =
+            MolHash::MolHash(&cp, MolHash::HashFunction::HetAtomProtomerv2);
+        CHECK(hsh != ref);
+      }
+    }
+  }
+}
+
+TEST_CASE("overreach with v2 tautomer hashes and imines") {
+  SECTION("basics") {
+    std::vector<std::string> smileses = {"C[C@H](F)NC1=CCCCC1",
+                                         "C[C@H](F)N=C1CCCCC1"};
+    for (const auto &smiles : smileses) {
+      auto m = v2::SmilesParse::MolFromSmiles(smiles);
+      REQUIRE(m);
+      auto hsh =
+          MolHash::MolHash(m.get(), MolHash::HashFunction::HetAtomTautomerv2);
+      CHECK(hsh ==
+            "[CH3]-[C@H](-[F])-[N]:[C]1:[C]-[CH2]-[CH2]-[CH2]-[C]:1_4_0");
+    }
+  }
+}
\ No newline at end of file
