@@ -19,12 +19,12 @@
 #include <iostream>
 #include <map>
 #include <regex>
-#include <set>
 #include <stdexcept>
 #include <unordered_set>
 #include <vector>
 
 #include <boost/dynamic_bitset.hpp>
+#include <boost/algorithm/string.hpp>
 
 #include <GraphMol/ROMol.h>
 #include <GraphMol/MolOps.h>
@@ -83,10 +83,10 @@ struct RascalStartPoint {
   // Some parts require mol2 to be the larger molecule.  This records if they
   // have been swapped with respect to the input molecules.
   bool d_swapped{false};
-  // Pointers the input molecules, swapped if necessary, so that d_mol1 is
-  // always the smaller molecule.  These should never be deleted.
-  const ROMol *d_mol1;
-  const ROMol *d_mol2;
+  // Pointers to copies of the input molecules, swapped if necessary,
+  // so that d_mol1 is always the smaller molecule.
+  std::unique_ptr<ROMol> d_mol1;
+  std::unique_ptr<ROMol> d_mol2;
 
   std::vector<std::vector<int>> d_adjMatrix1, d_adjMatrix2;
   std::vector<std::pair<int, int>> d_vtxPairs;
@@ -117,11 +117,17 @@ void sortedDegreeSeqs(
 }
 
 // Make labels for the atoms - by default the atomic symbol.
-void getAtomLabels(const ROMol &mol, const RascalOptions & /* opts */,
+void getAtomLabels(const ROMol &mol, const RascalOptions &opts,
                    std::vector<std::string> &atomLabels) {
   atomLabels.resize(mol.getNumAtoms());
   for (const auto &a : mol.atoms()) {
     std::string label = a->getSymbol();
+    if (opts.exactConnectionsMatch) {
+      label += "X" + std::to_string(a->getDegree());
+    }
+    if (!opts.ignoreAtomAromaticity && a->getIsAromatic()) {
+      label = "a" + label;
+    }
     atomLabels[a->getIdx()] = label;
   }
 }
@@ -253,14 +259,28 @@ void getBondLabels(const ROMol &mol, const RascalOptions &opts,
   getAtomLabels(mol, opts, atomLabels);
   bondLabels = std::vector<std::string>(mol.getNumBonds());
   for (const auto &b : mol.bonds()) {
-    if (b->getBeginAtom()->getAtomicNum() < b->getEndAtom()->getAtomicNum()) {
-      bondLabels[b->getIdx()] = atomLabels[b->getBeginAtomIdx()] +
-                                std::to_string(b->getBondType()) +
-                                atomLabels[b->getEndAtomIdx()];
+    std::string bondType;
+    if (opts.ignoreBondOrders) {
+      bondType = "0";
     } else {
-      bondLabels[b->getIdx()] = atomLabels[b->getEndAtomIdx()] +
-                                std::to_string(b->getBondType()) +
-                                atomLabels[b->getBeginAtomIdx()];
+      bondType = std::to_string(b->getBondType());
+    }
+    if (b->getBeginAtom()->getAtomicNum() == b->getEndAtom()->getAtomicNum()) {
+      if (b->getEndAtom()->getDegree() < b->getBeginAtom()->getDegree()) {
+        bondLabels[b->getIdx()] = atomLabels[b->getEndAtomIdx()] + bondType +
+                                  atomLabels[b->getBeginAtomIdx()];
+      } else {
+        bondLabels[b->getIdx()] = atomLabels[b->getBeginAtomIdx()] + bondType +
+                                  atomLabels[b->getEndAtomIdx()];
+      }
+    } else {
+      if (b->getBeginAtom()->getAtomicNum() < b->getEndAtom()->getAtomicNum()) {
+        bondLabels[b->getIdx()] = atomLabels[b->getBeginAtomIdx()] + bondType +
+                                  atomLabels[b->getEndAtomIdx()];
+      } else {
+        bondLabels[b->getIdx()] = atomLabels[b->getEndAtomIdx()] + bondType +
+                                  atomLabels[b->getBeginAtomIdx()];
+      }
     }
   }
 }
@@ -330,8 +350,17 @@ bool checkAromaticRings(const ROMol &mol1,
   if (!mol1Bond->getIsAromatic() || !mol2Bond->getIsAromatic()) {
     return true;
   }
+
+  // If neither bond was in a ring, but they were marked aromatic, then
+  // the two mols are fragments so it's ok to match these bonds.
   const auto &mol1BondRings = mol1.getRingInfo()->bondRings();
   const auto &mol2BondRings = mol2.getRingInfo()->bondRings();
+  bool mol1BondInRing = mol1.getRingInfo()->numBondRings(mol1BondIdx);
+  bool mol2BondInRing = mol2.getRingInfo()->numBondRings(mol2BondIdx);
+  if (!mol1BondInRing && !mol2BondInRing) {
+    return true;
+  }
+
   for (size_t i = 0u; i < mol1BondRings.size(); ++i) {
     if (std::find(mol1BondRings[i].begin(), mol1BondRings[i].end(),
                   mol1BondIdx) == mol1BondRings[i].end()) {
@@ -399,7 +428,8 @@ bool checkRingMatchesRing(const ROMol &mol1, int mol1BondIdx, const ROMol &mol2,
   return true;
 }
 
-// Make the set of pairs of vertices, where they're a pair if the labels match.
+// Make the set of pairs of vertices, where they're a pair if the labels
+// match.
 void buildPairs(const ROMol &mol1, const std::vector<unsigned int> &vtxLabels1,
                 const ROMol &mol2, const std::vector<unsigned int> &vtxLabels2,
                 const RascalOptions &opts,
@@ -407,7 +437,8 @@ void buildPairs(const ROMol &mol1, const std::vector<unsigned int> &vtxLabels1,
   std::vector<std::string> mol1RingSmiles, mol2RingSmiles;
   std::vector<std::unique_ptr<ROMol>> mol1Rings, mol2Rings;
   // For these purposes, it is correct that n1cccc1 and [nH]1cccc1 match - the
-  // former would be from an N-substituted pyrrole, the latter from a plain one.
+  // former would be from an N-substituted pyrrole, the latter from a plain
+  // one.
   static const std::regex reg(R"(\[([np])H\])");
   if (opts.completeAromaticRings) {
     extractRings(mol1, mol1Rings, mol1RingSmiles);
@@ -640,7 +671,8 @@ RWMol *makeCliqueFrags(const ROMol &mol,
   return molFrags;
 }
 
-// Calculate the shortest bond distance between the 2 fragments in the molecule.
+// Calculate the shortest bond distance between the 2 fragments in the
+// molecule.
 int minFragSeparation(const ROMol &mol, const ROMol &molFrags,
                       std::vector<int> &fragMapping, int frag1, int frag2) {
   auto extractFragAtoms = [&](int fragNum, std::vector<int> &fragAtoms) {
@@ -738,7 +770,8 @@ void updateMaxClique(const std::vector<unsigned int> &clique, bool deltaYPoss,
   }
 }
 
-// If the current time is beyond the timeout limit, throws a TimedOutException.
+// If the current time is beyond the timeout limit, throws a
+// TimedOutException.
 void checkTimeout(
     const std::chrono::time_point<std::chrono::high_resolution_clock>
         &startTime,
@@ -904,8 +937,8 @@ void explorePartitions(
 bool deltaYExchangePossible(const ROMol &mol1, const ROMol &mol2) {
   // A Delta-y exchange is an incorrect match when a cyclopropyl ring (the
   // delta) is matched to a C(C)(C) group (the y) because they both have
-  // isomorphic line graphs.  This checks to see if that's something we need to
-  // worry about for these molecules.
+  // isomorphic line graphs.  This checks to see if that's something we need
+  // to worry about for these molecules.
   const static std::unique_ptr<ROMol> delta(SmartsToMol("C1CC1"));
   const static std::unique_ptr<ROMol> y(SmartsToMol("C(C)C"));
   return (hasSubstructMatch(mol1, *delta) && hasSubstructMatch(mol2, *y)) ||
@@ -968,18 +1001,54 @@ void calcDistMatrix(const std::vector<std::vector<int>> &adjMatrix,
   }
 }
 
+// Set the atomic number of the atoms that match the SMARTS in
+// RascalOptions.EquivalentAtoms to 110, 111 etc.  These will
+// be mapped back at the end.
+void assignEquivalentAtoms(ROMol &mol, const std::string &equivalentAtoms) {
+  if (equivalentAtoms.empty()) {
+    return;
+  }
+  std::vector<std::string> classSmarts;
+  boost::split(classSmarts, equivalentAtoms, boost::is_any_of(" "),
+               boost::token_compress_on);
+  if (classSmarts.size() > 9) {
+    throw ValueErrorException(
+        "Too many classes of equivalent atoms.  Maximum is 9.");
+  }
+  int atNum = 110;
+  for (auto &smt : classSmarts) {
+    if (smt.empty()) {
+      continue;
+    }
+    auto qmol = v2::SmilesParse::MolFromSmarts(smt);
+    std::vector<RDKit::MatchVectType> hits_vect;
+    if (RDKit::SubstructMatch(mol, *qmol, hits_vect)) {
+      for (const auto &hv : hits_vect) {
+        for (const auto &h : hv) {
+          auto a = mol.getAtomWithIdx(h.second);
+          a->setAtomicNum(atNum);
+        }
+      }
+    }
+    ++atNum;
+  }
+}
+
 RascalStartPoint makeInitialPartitionSet(const ROMol *mol1, const ROMol *mol2,
                                          const RascalOptions &opts) {
   RascalStartPoint starter;
   if (mol1->getNumAtoms() <= mol2->getNumAtoms()) {
     starter.d_swapped = false;
-    starter.d_mol1 = mol1;
-    starter.d_mol2 = mol2;
+    starter.d_mol1.reset(new ROMol(*mol1));
+    starter.d_mol2.reset(new ROMol(*mol2));
   } else {
     starter.d_swapped = true;
-    starter.d_mol1 = mol2;
-    starter.d_mol2 = mol1;
+    starter.d_mol1.reset(new ROMol(*mol2));
+    starter.d_mol2.reset(new ROMol(*mol1));
   }
+  assignEquivalentAtoms(*starter.d_mol1, opts.equivalentAtoms);
+  assignEquivalentAtoms(*starter.d_mol2, opts.equivalentAtoms);
+
   std::map<int, std::vector<std::pair<int, int>>> degSeqs1, degSeqs2;
   starter.d_tier1Sim =
       details::tier1Sim(*starter.d_mol1, *starter.d_mol2, degSeqs1, degSeqs2);
@@ -1063,14 +1132,15 @@ std::vector<RascalResult> findMCES(RascalStartPoint &starter,
                      starter.d_adjMatrix2, c, starter.d_vtxPairs, timedOut,
                      starter.d_swapped, starter.d_tier1Sim, starter.d_tier2Sim,
                      opts.ringMatchesRingOnly, opts.singleLargestFrag,
-                     opts.maxFragSeparation));
+                     opts.maxFragSeparation, opts.exactConnectionsMatch,
+                     opts.equivalentAtoms, opts.ignoreBondOrders));
   }
   std::sort(results.begin(), results.end(), details::resultCompare);
   return results;
 }
 
-// calculate the RASCAL MCES between the 2 molecules, provided it is within the
-// similarity threshold given.
+// calculate the RASCAL MCES between the 2 molecules, provided it is within
+// the similarity threshold given.
 std::vector<RascalResult> rascalMCES(const ROMol &mol1, const ROMol &mol2,
                                      const RascalOptions &opts) {
   auto starter = makeInitialPartitionSet(&mol1, &mol2, opts);
@@ -1078,11 +1148,14 @@ std::vector<RascalResult> rascalMCES(const ROMol &mol1, const ROMol &mol2,
     if (opts.returnEmptyMCES) {
       return std::vector<RascalResult>(
           1, RascalResult(starter.d_tier1Sim, starter.d_tier2Sim));
-    } else {
-      return std::vector<RascalResult>();
     }
+    return std::vector<RascalResult>();
   }
   auto results = findMCES(starter, opts);
+  if (results.empty() && opts.returnEmptyMCES) {
+    return std::vector<RascalResult>(
+        1, RascalResult(starter.d_tier1Sim, starter.d_tier2Sim));
+  }
   if (!opts.allBestMCESs && results.size() > 1) {
     results.erase(results.begin() + 1, results.end());
   }
