@@ -36,7 +36,7 @@ using namespace RDKit::RascalMCES::details;
 
 void check_smarts_ok(const RDKit::ROMol &mol1, const RDKit::ROMol &mol2,
                      const RascalResult &res) {
-  std::unique_ptr<RDKit::ROMol> qmol(RDKit::SmartsToMol(res.getSmarts()));
+  auto qmol = v2::SmilesParse::MolFromSmarts(res.getSmarts());
   RDKit::MatchVectType dont_care;
   REQUIRE(RDKit::SubstructMatch(mol1, *qmol, dont_care));
   REQUIRE(RDKit::SubstructMatch(mol2, *qmol, dont_care));
@@ -1207,3 +1207,238 @@ TEST_CASE("Trim small frags") {
   REQUIRE(res.front().getNumFrags() == 2);
   REQUIRE(res.front().getSmarts() == "CCCC(=O)-NCCC-[#6].Cc1:c:c:c:c:c:1");
 }
+
+TEST_CASE("Exact Connection Matches", "[basics]") {
+  {
+    auto m1 = "c1ccccc1OC(=O)C(N)N"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1ccccc1SC(=O)C2NCC(CN2)C(=O)C(N)N"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.1;
+    opts.allBestMCESs = true;
+    auto res1 = rascalMCES(*m1, *m2, opts);
+    CHECK(res1.front().getNumFrags() == 2);
+    CHECK(res1.front().getSmarts() == "c1:c:c:c:c:c:1.C(=O)-C(-N)-N");
+    auto q1 = v2::SmilesParse::MolFromSmarts(res1.front().getSmarts());
+    std::vector<MatchVectType> smarts_res;
+    SubstructMatch(*m1, *q1, smarts_res);
+    CHECK(smarts_res.size() == 1);
+    // There are 2 equivalent MCESs for the 2nd molecule, so 2 SMARTS matches.
+    SubstructMatch(*m2, *q1, smarts_res);
+    CHECK(smarts_res.size() == 2);
+
+    opts.exactConnectionsMatch = true;
+    auto res2 = rascalMCES(*m1, *m2, opts);
+    CHECK(res2.front().getNumFrags() == 2);
+    CHECK(res2.front().getSmarts() ==
+          "[#6&a&D2]1:[#6&a&D2]:[#6&a&D2]:[#6&a&D2]:[#6&a&D2]:[#6&a&D3]:1."
+          "[#6&A&D3](=[#8&A&D1])-[#6&A&D3](-[#7&A&D1])-[#7&A&D1]");
+    auto q2 = v2::SmilesParse::MolFromSmarts(res2.front().getSmarts());
+    SubstructMatch(*m1, *q2, smarts_res);
+    CHECK(smarts_res.size() == 1);
+    // With the exactConnectionsMatch option, only the amide group
+    // furthest from the phenyl is a match.
+    SubstructMatch(*m2, *q2, smarts_res);
+    CHECK(smarts_res.size() == 1);
+  }
+  {
+    // The original implementation had a crappy order-dependence bug such
+    // that m1, m2 gave 12 bonds in the MCES but m3, m2 gave 10.
+    // They are both the same molecules.
+    auto m1 = "c1cccc(Cc2cccnc2)c1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1ncccc1Oc1ccccc1"_smiles;
+    REQUIRE(m2);
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.allBestMCESs = false;
+    opts.exactConnectionsMatch = true;
+
+    auto res1 = rascalMCES(*m1, *m2, opts);
+    CHECK(res1.front().getBondMatches().size() == 12);
+
+    auto m3 = "c1ncccc1Cc1ccccc1"_smiles;
+    REQUIRE(m3);
+
+    auto res2 = rascalMCES(*m3, *m2, opts);
+    CHECK(res2.front().getBondMatches().size() == 12);
+
+    // and try really hard to break it.
+    SmilesWriteParams params;
+    params.doRandom = true;
+    for (int i = 0; i < 100; ++i) {
+      auto m3 = v2::SmilesParse::MolFromSmiles(MolToSmiles(*m1, params));
+      REQUIRE(m3);
+      auto m4 = v2::SmilesParse::MolFromSmiles(MolToSmiles(*m2, params));
+      REQUIRE(m4);
+      auto res2 = rascalMCES(*m3, *m4, opts);
+      CHECK(res2.front().getBondMatches().size() == 12);
+    }
+  }
+}
+
+TEST_CASE("Equivalent atoms") {
+  {
+    auto m1 = "c1cc(Br)ccc1F"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1cc(I)ccc1Cl"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.equivalentAtoms = "[F,Cl,Br,I]";
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 8);
+    CHECK(res.front().getSmarts() ==
+          "c1:c:c(-[F,Cl,Br,I]):c:c:c:1-[F,Cl,Br,I]");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+  {
+    auto m1 = "c1cc(Br)ccc1F"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1ncccc1Cl"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.equivalentAtoms = "[F,Cl,Br,I] [c,n]";
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 7);
+    CHECK(res.front().getSmarts() ==
+          "[c,n]1:[c,n]:[c,n]:[c,n]:[c,n]:[c,n]:1-[F,Cl,Br,I]");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+  {
+    auto m1 = "c1ccccc1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1nc(I)ccc1Cl"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.equivalentAtoms = "[*]";
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 6);
+    CHECK(res.front().getSmarts() == "[*]1:[*]:[*]:[*]:[*]:[*]:1");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+  {
+    auto m1 = "c1ccccc1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1nc(I)ccc1Cl"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    // Nonsense options, to check that too many exits correctly.
+    opts.equivalentAtoms = "[*] [*] [*] [*] [*] [*] [*] [*] [*] [*] [*] ";
+    CHECK_THROWS_AS(rascalMCES(*m1, *m2, opts), ValueErrorException);
+  }
+}
+
+TEST_CASE("Equivalent bonds") {
+  {
+    auto m1 = "CC=CC"_smiles;
+    REQUIRE(m1);
+    auto m2 = "CCCC"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.ignoreBondOrders = true;
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 4);
+    CHECK(res.front().getBondMatches().size() == 3);
+    CHECK(res.front().getSmarts() == "C~C~C~C");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+  {
+    auto m1 = "C1NC(C(=O)O)CCC1"_smiles;
+    REQUIRE(m1);
+    auto m2 = "c1ccnc(C(=O)O)c1"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.5;
+    opts.equivalentAtoms = "[#6,#7]";
+    opts.ignoreBondOrders = true;
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 9);
+    CHECK(res.front().getBondMatches().size() == 9);
+    CHECK(res.front().getSmarts() ==
+          "[#6,#7]1~[#6,#7]~[#6,#7](~[#6,#7](~O)~O)~[#6,#7]~[#6,#7]~[#6,#7]~1");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+}
+
+TEST_CASE("Atom aromaticity match") {
+  {
+    auto m1 = "c1ccccc1NCC"_smiles;
+    REQUIRE(m1);
+    auto m2 = "C1CCCCC1NCC"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.1;
+    opts.ignoreAtomAromaticity = true;
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 4);
+    CHECK(res.front().getBondMatches().size() == 3);
+    CHECK(res.front().getSmarts() == "[#6]-NCC");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+  {
+    auto m1 = "c1ccccc1NCC"_smiles;
+    REQUIRE(m1);
+    auto m2 = "C1CCCCC1NCC"_smiles;
+    REQUIRE(m2);
+
+    RascalOptions opts;
+    opts.similarityThreshold = 0.1;
+    opts.ignoreAtomAromaticity = false;
+    auto res = rascalMCES(*m1, *m2, opts);
+    REQUIRE(res.size() == 1);
+    CHECK(res.front().getAtomMatches().size() == 3);
+    CHECK(res.front().getBondMatches().size() == 2);
+    CHECK(res.front().getSmarts() == "NCC");
+    check_smarts_ok(*m1, *m2, res.front());
+  }
+}
+
+TEST_CASE("Aromatic fragment match") {
+  v2::SmilesParse::SmilesParserParams params;
+  params.sanitize = false;
+
+  auto m1 = v2::SmilesParse::MolFromSmiles("[1*]c([3*])nc[2*]", params);
+  REQUIRE(m1);
+  auto m2 = v2::SmilesParse::MolFromSmiles("[1*]c([3*])nc[2*]", params);
+  REQUIRE(m2);
+
+  RascalOptions opts;
+  auto res = rascalMCES(*m1, *m2, opts);
+  REQUIRE(res.size() == 1);
+  CHECK(res.front().getAtomMatches().size() == 6);
+  CHECK(res.front().getBondMatches().size() == 5);
+}
+
+TEST_CASE("Empty results bug") {
+  // These 2 molecules gave an empty results object even
+  // when told not to.
+  auto m1 = "Cc1nnc(NC(=O)C(=O)NCCc2c[nH]c3ccccc23)s1"_smiles;
+  REQUIRE(m1);
+  auto m2 = "Cc1ccc2c(C(=O)NCc3cncs3)n[nH]c2c1"_smiles;
+  REQUIRE(m2);
+
+  RascalOptions opts;
+  opts.returnEmptyMCES = true;
+  auto res = rascalMCES(*m1, *m2, opts);
+  REQUIRE(!res.empty());
+}
