@@ -18,7 +18,7 @@
 #include <GraphMol/RDKitBase.h>
 #include <GraphMol/MolPickler.h>
 #include <RDGeneral/RDLog.h>
-//#include <boost/log/functions.hpp>
+// #include <boost/log/functions.hpp>
 using namespace RDKit;
 using namespace std;
 typedef ROMol Mol;
@@ -31,108 +31,47 @@ void testPass() {
   BOOST_LOG(rdInfoLog) << "-------------------------------------" << std::endl;
   BOOST_LOG(rdInfoLog) << "Testing patterns which should parse." << std::endl;
   string smis[] = {
-#if 1
-    "C",
-    "CC",
-    "C-C",
-    "C=C",
-    "[CH2+]C[CH+2]",
-    "C1CC=1",
-    "C=1CC1",
-    "Ccc",
-    "C=C-O",
-    "C1CC1",
-    "C1NC1",
-    "C1=CC1",
-    "C1CCC1",
-    "CC(C)CC",
-    "CC(=O)O",
-    "C1C(=O)C1",
-    "C1C(N)C1",
-    "CC(O)C",
-    "OC=CCC",
-    "CC([O-])O",
-    "C1CC2C1CC2",
-    "Cl/C=C/Cl",
-    "Cl/C=C\\Cl",
-    "Cl/C=C/Cl",
-    "Cl/C=C\\Cl",
-    "C1CC.CC1",
-    "C1C(C2CC2).C2CC2C1",
-    "[Na+].[Cl-].[NH4+].[Cl-]",
-    "[$(CO)]CO",
-    "[!$(*#*)]",
-    "[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]",
-    "[C;!$(C-[OH])]=O",
-    "[CH2;X4][N,O,S,F,Cl,Br,I]",
-    "C=O",
-    "[C;!$(C-[OH])]=O",
-    "[#6]-!:[#6]",
-    "[C^3]",
-    "[*^0]",
-    "[*^1]",
-    "[*^2]",
-    "[*^4]",
-    "[*^5]",
-    "[se]",
-    "[te]",
-    // test zeros as ring indices, issue 2690982:
-    "C0CC0",
-    // these used to fail before Roger Sayle's SMARTS patch:
-    "[HO]",
-    "[13]",
-    "[+]",
-    "[C:1]",
-    "[C:0]",           // issue 3525776
-    "[$([#0].[#0])]",  // sf.net issue 281
-    //"CC((C)C)", // github #102
-    "c1ccccb1",                                            // github 220
-    "[Db][Sg][Bh][Hs][Mt][Ds][Rg][Cn][Uut][Fl][Uup][Lv]",  // new elements
-    "C->[Cu]<-C",                                          // dative bonds
-    "C%(1)CC%(1)",          // high ring closures (Github #1624)
-    "C%(10)CC%(10)",        // high ring closures (Github #1624)
-    "C%(100)CC%(100)",      // high ring closures (Github #1624)
-    "C%(1000)CC%(1000)",    // high ring closures (Github #1624)
-    "C%(10000)CC%(10000)",  // high ring closures (Github #1624)
-    "[z]",                  // cactvs heteroatom neighbor queries
-    "[z1]",
-    "[Z]",
-    "[Z1]",
-    "[D{1-3}]",  // cactvs range queries
-    "[D{-3}]",
-    "[D{1-}]",
-    "[z{1-3}]",
-    "[Z{1-3}]",
-    "[2H,13C]",  // github #1719
-    "[+{0-3}]",
-#endif
-    "[-{0-3}]",
-    "[-{0-3},C]",
-    "[-{0-3},D{1-3}]",       // github #2709
-    "C%(1000)CCC%(1000)",    // github #2909
-    "C%(1000)CC(C%(1000))",  // github #2909
-    "C%(1000)CC.C%(1000)",   // github #2909
-    "[C;d2]",                // non-hydrogen degree
-    "C$C",                   // quadruple bonds
-    // extended chirality
-    "C[Fe@TH](O)(Cl)F",
-    "C[Fe@TH1](O)(Cl)F",
-    "C[Fe@SP](O)(Cl)F",
-    "C[Fe@SP1](O)(Cl)F",
-    "C[Fe@TB](O)(Cl)(Br)F",
-    "C[Fe@TB20](O)(Cl)(Br)F",
-    "C[Fe@OH](O)(Cl)(Br)(N)F",
-    "C[Fe@OH20](O)(Cl)(Br)(N)F",
-    "[@TH]",
-    "[@TH1]",
-    "[@SP]",
-    "[@SP1]",
-    "[@TB]",
-    "[@TB10]",
-    "[@OH]",
-    "[@OH20]",
-    "EOS"
-  };
+      "C", "CC", "C-C", "C=C", "[CH2+]C[CH+2]", "C1CC=1", "C=1CC1", "Ccc",
+      "C=C-O", "C1CC1", "C1NC1", "C1=CC1", "C1CCC1", "CC(C)CC", "CC(=O)O",
+      "C1C(=O)C1", "C1C(N)C1", "CC(O)C", "OC=CCC", "CC([O-])O", "C1CC2C1CC2",
+      "Cl/C=C/Cl", "Cl/C=C\\Cl", "Cl/C=C/Cl", "Cl/C=C\\Cl", "C1CC.CC1",
+      "C1C(C2CC2).C2CC2C1", "[Na+].[Cl-].[NH4+].[Cl-]", "[$(CO)]CO",
+      "[!$(*#*)]", "[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]", "[C;!$(C-[OH])]=O",
+      "[CH2;X4][N,O,S,F,Cl,Br,I]", "C=O", "[C;!$(C-[OH])]=O", "[#6]-!:[#6]",
+      "[C^3]", "[*^0]", "[*^1]", "[*^2]", "[*^4]", "[*^5]", "[se]", "[te]",
+      // test zeros as ring indices, issue 2690982:
+      "C0CC0",
+      // these used to fail before Roger Sayle's SMARTS patch:
+      "[HO]", "[13]", "[+]", "[C:1]",
+      "[C:0]",           // issue 3525776
+      "[$([#0].[#0])]",  // sf.net issue 281
+      //"CC((C)C)", // github #102
+      "c1ccccb1",                                            // github 220
+      "[Db][Sg][Bh][Hs][Mt][Ds][Rg][Cn][Uut][Fl][Uup][Lv]",  // new elements
+      "C->[Cu]<-C",                                          // dative bonds
+      "C%(1)CC%(1)",          // high ring closures (Github #1624)
+      "C%(10)CC%(10)",        // high ring closures (Github #1624)
+      "C%(100)CC%(100)",      // high ring closures (Github #1624)
+      "C%(1000)CC%(1000)",    // high ring closures (Github #1624)
+      "C%(10000)CC%(10000)",  // high ring closures (Github #1624)
+      "[z]",                  // cactvs heteroatom neighbor queries
+      "[z1]", "[Z]", "[Z1]",
+      "[D{1-3}]",  // cactvs range queries
+      "[D{-3}]", "[D{1-}]", "[z{1-3}]", "[Z{1-3}]",
+      "[2H,13C]",  // github #1719
+      "[+{0-3}]",
+      "[-{0-3}]", "[-{0-3},C]",
+      "[-{0-3},D{1-3}]",       // github #2709
+      "C%(1000)CCC%(1000)",    // github #2909
+      "C%(1000)CC(C%(1000))",  // github #2909
+      "C%(1000)CC.C%(1000)",   // github #2909
+      "[C;d2]",                // non-hydrogen degree
+      "C$C",                   // quadruple bonds
+      // extended chirality
+      "C[Fe@TH](O)(Cl)F", "C[Fe@TH1](O)(Cl)F", "C[Fe@SP](O)(Cl)F",
+      "C[Fe@SP1](O)(Cl)F", "C[Fe@TB](O)(Cl)(Br)F", "C[Fe@TB20](O)(Cl)(Br)F",
+      "C[Fe@OH](O)(Cl)(Br)(N)F", "C[Fe@OH20](O)(Cl)(Br)(N)F", "[@TH]", "[@TH1]",
+      "[@SP]", "[@SP1]", "[@TB]", "[@TB10]", "[@OH]", "[@OH20]", "EOS"};
   while (smis[i] != "EOS") {
     string smi = smis[i];
     mol = SmartsToMol(smi);
@@ -492,7 +431,6 @@ void testProblems() {
   BOOST_LOG(rdInfoLog) << "-------------------------------------" << std::endl;
   BOOST_LOG(rdInfoLog) << "Testing former problems" << std::endl;
 
-#if 1
   _checkMatches("[$(C(=O)O)]", "CC(=O)OC", 1, 1);
 
   _checkMatches("[$(C(=O)[O,N])]", "CC(=O)OC", 1, 1);
@@ -537,7 +475,6 @@ void testProblems() {
   _checkNoMatches("[O]-[!$(*=O)]", "CC(=O)O");
 
 // BOOST_LOG(rdInfoLog) << "-*-*-*-*-*-*-*-*-" << std::endl;
-#endif
   _checkNoMatches("[$([O]-[!$(*=O)])]", "CC(=O)O");
 
   // ISSUE 78
@@ -897,53 +834,6 @@ void testIssue254() {
   BOOST_LOG(rdInfoLog) << "\tdone" << std::endl;
 }
 
-void testIssue255() {
-  ROMol *matcher1;
-  std::string sma;
-  BOOST_LOG(rdInfoLog) << "-------------------------------------" << std::endl;
-  BOOST_LOG(rdInfoLog) << "Testing Issue 255: Core leaks in smarts parsing.  "
-                          "Watch memory consumption."
-                       << std::endl;
-
-  for (int i = 0; i < 10000; i++) {
-#if 1
-    sma = "CC";
-    matcher1 = SmartsToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-
-    sma = "C-C";
-    matcher1 = SmartsToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-
-    sma = "C=C";
-    matcher1 = SmartsToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-
-    sma = "C=,#C";
-    matcher1 = SmartsToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-#endif
-    sma = "C-1CC-1";
-    // matcher1 = SmartsToMol(sma);
-    matcher1 = SmilesToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-#if 1
-
-    sma = "C-1CC1";
-    matcher1 = SmartsToMol(sma);
-    TEST_ASSERT(matcher1);
-    delete matcher1;
-#endif
-  }
-
-  BOOST_LOG(rdInfoLog) << "\tdone" << std::endl;
-}
-
 void testIssue330() {
   ROMol *matcher1;
   std::string sma, wsma;
@@ -1046,7 +936,6 @@ void testAtomMap() {
   BOOST_LOG(rdInfoLog) << "\tdone" << std::endl;
 }
 
-#if 1
 void testIssue1804420() {
   ROMol *matcher1;
   std::string sma;
@@ -1084,7 +973,6 @@ void testIssue1804420() {
 
   BOOST_LOG(rdInfoLog) << "\tdone" << std::endl;
 }
-#endif
 
 void testSmartsSmiles() {
   RWMol *mol;
@@ -1146,14 +1034,16 @@ void testSmilesSmarts() {
   delete mol;
 
   smi = "[CH4+]C";
-  mol = SmilesToMol(smi);
+  auto debugParse = false;
+  auto sanitize = false;
+  mol = SmilesToMol(smi, debugParse, sanitize);
   TEST_ASSERT(mol);
   sma = MolToSmarts(*mol);
   TEST_ASSERT(sma == "[#6H4+]-[#6]");
   delete mol;
 
   smi = "[CH5+2]C";
-  mol = SmilesToMol(smi);
+  mol = SmilesToMol(smi, debugParse, sanitize);
   TEST_ASSERT(mol);
   sma = MolToSmarts(*mol);
   TEST_ASSERT(sma == "[#6H5+2]-[#6]");
@@ -2837,13 +2727,13 @@ void testRingBondCrash() {
 
 void testGithub6730() {
   BOOST_LOG(rdInfoLog) << "-------------------------------------" << std::endl;
-  BOOST_LOG(rdInfoLog)
-      << "Testing stereo inverts on first atom of SMARTS" << std::endl;
+  BOOST_LOG(rdInfoLog) << "Testing stereo inverts on first atom of SMARTS"
+                       << std::endl;
   {
-    auto input    = "[C@H](F)(Cl)O"_smarts;
+    auto input = "[C@H](F)(Cl)O"_smarts;
     auto expected = "F[C@@&H1](Cl)O";
-    auto actual   = MolToSmarts(*input,true,1); // doChiral,root=F(1)
-    TEST_ASSERT(expected == actual);    
+    auto actual = MolToSmarts(*input, true, 1);  // doChiral,root=F(1)
+    TEST_ASSERT(expected == actual);
   }
 
   BOOST_LOG(rdInfoLog) << "done" << std::endl;
@@ -2853,7 +2743,6 @@ int main(int argc, char *argv[]) {
   (void)argc;
   (void)argv;
   RDLog::InitLogs();
-#if 1
   testPass();
   testFail();
   testMatches();
@@ -2867,8 +2756,6 @@ int main(int argc, char *argv[]) {
   testProblems();
   testIssue196();
   testIssue254();
-  // testIssue255(); // this is a slow one and doesn't really actually test much
-  // without someone watching memory consumption
   testIssue330();
   testIssue351();
   testAtomMap();
@@ -2904,7 +2791,6 @@ int main(int argc, char *argv[]) {
   testGithub2565();
   testSmartsStereoBonds();
   testGithub6730();
-#endif
   testRingBondCrash();
   return 0;
 }
