@@ -175,7 +175,7 @@ void ResSubstructMatchHelper_(const ResSubstructMatchHelperArgs_ &args,
                               std::set<MatchVectType> *matches, unsigned int bi,
                               unsigned int ei);
 
-typedef std::list<
+typedef std::vector<
     std::pair<MolGraph::vertex_descriptor, MolGraph::vertex_descriptor>>
     ssPairType;
 
@@ -201,9 +201,6 @@ bool MolMatchFinalCheckFunctor::operator()(const std::uint32_t q_c[],
   if (d_params.extraFinalCheck || d_params.useGenericMatchers) {
     // EFF: we can no-doubt do better than this
     std::vector<unsigned int> aids(m_c, m_c + d_query.getNumAtoms());
-    for (unsigned int i = 0; i < d_query.getNumAtoms(); ++i) {
-      aids[i] = m_c[i];
-    }
     if (d_params.useGenericMatchers &&
         !GenericGroups::genericAtomMatcher(d_mol, d_query, aids)) {
       return false;
@@ -249,6 +246,9 @@ bool MolMatchFinalCheckFunctor::operator()(const std::uint32_t q_c[],
     }
     const Atom *mAt = d_mol.getAtomWithIdx(m_c[i]);
     if (!detail::hasChiralLabel(mAt)) {
+      if (d_params.specifiedStereoQueryMatchesUnspecified) {
+        continue;
+      }
       return false;
     }
     if (qAt->getDegree() > mAt->getDegree()) {
@@ -330,10 +330,15 @@ bool MolMatchFinalCheckFunctor::operator()(const std::uint32_t q_c[],
     const Bond *mBnd = d_mol.getBondBetweenAtoms(
         q_to_mol[qBnd->getBeginAtomIdx()], q_to_mol[qBnd->getEndAtomIdx()]);
     CHECK_INVARIANT(mBnd, "Matching bond not found");
-    if (mBnd->getBondType() != Bond::DOUBLE ||
-        qBnd->getStereo() <= Bond::STEREOANY) {
+    if (mBnd->getBondType() != Bond::DOUBLE) {
       continue;
     }
+
+    if (!d_params.specifiedStereoQueryMatchesUnspecified &&
+        mBnd->getStereo() <= Bond::STEREOANY) {
+      return false;
+    }
+
     // don't think this can actually happen, but check to be sure:
     if (mBnd->getStereoAtoms().size() != 2) {
       continue;
@@ -388,7 +393,8 @@ class AtomLabelFunctor {
  public:
   AtomLabelFunctor(const ROMol &query, const ROMol &mol,
                    const SubstructMatchParameters &ps)
-      : d_query(query), d_mol(mol), d_params(ps){};
+      : d_query(query), d_mol(mol), d_params(ps) {};
+
   bool operator()(unsigned int i, unsigned int j) const {
     bool res = false;
     if (d_params.useChirality) {
@@ -396,7 +402,8 @@ class AtomLabelFunctor {
       if (qAt->getChiralTag() == Atom::CHI_TETRAHEDRAL_CW ||
           qAt->getChiralTag() == Atom::CHI_TETRAHEDRAL_CCW) {
         const Atom *mAt = d_mol.getAtomWithIdx(j);
-        if (mAt->getChiralTag() != Atom::CHI_TETRAHEDRAL_CW &&
+        if (!d_params.specifiedStereoQueryMatchesUnspecified &&
+            mAt->getChiralTag() != Atom::CHI_TETRAHEDRAL_CW &&
             mAt->getChiralTag() != Atom::CHI_TETRAHEDRAL_CCW) {
           return false;
         }
@@ -415,7 +422,7 @@ class BondLabelFunctor {
  public:
   BondLabelFunctor(const ROMol &query, const ROMol &mol,
                    const SubstructMatchParameters &ps)
-      : d_query(query), d_mol(mol), d_params(ps){};
+      : d_query(query), d_mol(mol), d_params(ps) {};
   bool operator()(MolGraph::edge_descriptor i,
                   MolGraph::edge_descriptor j) const {
     if (d_params.useChirality) {
@@ -424,6 +431,7 @@ class BondLabelFunctor {
           qBnd->getStereo() > Bond::STEREOANY) {
         const Bond *mBnd = d_mol[j];
         if (mBnd->getBondType() == Bond::DOUBLE &&
+            !d_params.specifiedStereoQueryMatchesUnspecified &&
             mBnd->getStereo() <= Bond::STEREOANY) {
           return false;
         }
@@ -503,7 +511,7 @@ std::vector<MatchVectType> SubstructMatch(
   detail::BondLabelFunctor bondLabeler(query, mol, params);
   MolMatchFinalCheckFunctor matchChecker(query, mol, params);
 
-  std::list<detail::ssPairType> pms;
+  std::vector<detail::ssPairType> pms;
 #if 0
   bool found=boost::ullmann_all(query.getTopology(),mol.getTopology(),
 				atomLabeler,bondLabeler,pms);
@@ -629,13 +637,14 @@ unsigned int RecursiveMatcher(const ROMol &mol, const ROMol &query,
 
   matches.clear();
   matches.resize(0);
-  std::list<detail::ssPairType> pms;
+  std::vector<detail::ssPairType> pms;
 #if 0
       bool found=boost::ullmann_all(query.getTopology(),mol.getTopology(),
 				    atomLabeler,bondLabeler,pms);
 #else
-  bool found = boost::vf2_all(query.getTopology(), mol.getTopology(),
-                              atomLabeler, bondLabeler, matchChecker, pms);
+  bool found =
+      boost::vf2_all(query.getTopology(), mol.getTopology(), atomLabeler,
+                     bondLabeler, matchChecker, pms, lparams.maxMatches);
 #endif
   unsigned int res = 0;
   if (found) {
