@@ -246,11 +246,10 @@ std::vector<MatchVectType> sortMatchesByDegreeOfCoreSubstitution(
 }
 
 bool isAtomTerminalRGroupOrQueryHydrogen(const Atom *atom) {
-  return atom->getDegree() == 1 &&
-         (atom->getAtomicNum() == 0 ||
-          (atom->hasQuery() &&
-           describeQuery(atom).find("AtomAtomicNum 1 = val") !=
-               std::string::npos));
+  return (atom->getDegree() == 1 && isAtomDummy(atom)) ||
+         (atom->hasQuery() &&
+          describeQuery(atom).find("AtomAtomicNum 1 = val") !=
+              std::string::npos);
 }
 
 #define PT_OPT_GET(opt) params.opt = pt.get(#opt, params.opt)
@@ -274,6 +273,7 @@ void updateSubstructMatchParamsFromJSON(SubstructMatchParameters &params,
   PT_OPT_GET(maxMatches);
   PT_OPT_GET(maxRecursiveMatches);
   PT_OPT_GET(numThreads);
+  PT_OPT_GET(specifiedStereoQueryMatchesUnspecified);
 }
 
 std::string substructMatchParamsToJSON(const SubstructMatchParameters &params) {
@@ -288,6 +288,7 @@ std::string substructMatchParamsToJSON(const SubstructMatchParameters &params) {
   PT_OPT_PUT(maxMatches);
   PT_OPT_PUT(maxRecursiveMatches);
   PT_OPT_PUT(numThreads);
+  PT_OPT_PUT(specifiedStereoQueryMatchesUnspecified);
 
   std::stringstream ss;
   boost::property_tree::json_parser::write_json(ss, pt);
