@@ -19,6 +19,8 @@
 #include <GraphMol/SmilesParse/SmilesParse.h>
 #include <GraphMol/SmilesParse/SmilesWrite.h>
 #include <GraphMol/Substruct/SubstructMatch.h>
+#include <GraphMol/QueryOps.h>
+#include <GraphMol/MolPickler.h>
 
 using namespace RDKit;
 typedef std::tuple<std::string, std::string, size_t> matchCase;
@@ -513,4 +515,273 @@ TEST_CASE("Github #6017: add maxRecursiveMatches to SubstructMatchParameters") {
       CHECK(matches.size() == 3);
     }
   }
+}
+
+TEST_CASE(
+    "GitHub Issue #6983: SubstructMatch maxRecursiveMatches is not being honored",
+    "[bug][substruct]") {
+  constexpr unsigned num_atoms = 1005;
+  std::string smiles;
+  smiles.reserve(num_atoms * 2);
+
+  // 'smiles' already contains 1 O, so start from 1
+  // so we end up with 'num_atoms' water mols
+  smiles += "O";
+  for (unsigned i = 1; i < num_atoms; ++i) {
+    smiles += ".O";
+  }
+  std::unique_ptr<RWMol> m(SmilesToMol(smiles));
+  auto q = "[$(O)]"_smarts;
+  REQUIRE(m);
+  REQUIRE(q);
+
+  SubstructMatchParameters ps;
+  ps.maxMatches = num_atoms * 2;
+  ps.maxRecursiveMatches = ps.maxMatches;
+  {
+    auto matches = SubstructMatch(*m, *q, ps);
+    CHECK(matches.size() == num_atoms);
+  }
+}
+
+TEST_CASE("pickling HasPropWithValue queries") {
+  SubstructMatchParameters ps;
+  SECTION("basics int") {
+    auto mol = "CC"_smarts;
+    auto target = "CC"_smiles;
+    REQUIRE(mol);
+    REQUIRE(target);
+
+    RWMol mol2(*mol);
+    RWMol mol3(*mol);
+
+    mol->getAtomWithIdx(0)->expandQuery(makePropQuery<Atom, int>("foo", 1, 1));
+    mol->getBondWithIdx(0)->expandQuery(makePropQuery<Bond, int>("bar", 1, 0));
+
+    mol2.getAtomWithIdx(0)->expandQuery(makePropQuery<Atom, int>("foo", 1, 0));
+    mol2.getBondWithIdx(0)->expandQuery(makePropQuery<Bond, int>("bar", 1, 0));
+    mol3.getAtomWithIdx(0)->expandQuery(makePropQuery<Atom, int>("foo", 2, 0));
+    mol3.getBondWithIdx(0)->expandQuery(makePropQuery<Bond, int>("bar", 2, 0));
+
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    target->getAtomWithIdx(0)->setProp<int>("foo", 2);
+    target->getBondWithIdx(0)->setProp<int>("bar", 1);
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+
+    {
+      std::string pkl;
+      MolPickler::pickleMol(*mol, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 1);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol2, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol3, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    }
+  }
+  SECTION("basics string") {
+    auto mol = "CC"_smarts;
+    auto target = "CC"_smiles;
+    REQUIRE(mol);
+    REQUIRE(target);
+
+    RWMol mol2(*mol);
+    RWMol mol3(*mol);
+
+    mol->getAtomWithIdx(0)->expandQuery(
+        makePropQuery<Atom, std::string>("foo", "asdfs"));
+    mol->getBondWithIdx(0)->expandQuery(
+        makePropQuery<Bond, std::string>("bar", "dsafasdf"));
+
+    mol2.getAtomWithIdx(0)->expandQuery(
+        makePropQuery<Atom, std::string>("foo", "asdfs"));
+    mol2.getBondWithIdx(0)->expandQuery(
+        makePropQuery<Bond, std::string>("bar", "dsa"));
+    mol3.getAtomWithIdx(0)->expandQuery(
+        makePropQuery<Atom, std::string>("foo", "asd"));
+    mol3.getBondWithIdx(0)->expandQuery(
+        makePropQuery<Bond, std::string>("bar", "dsafasdf"));
+
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    target->getAtomWithIdx(0)->setProp<std::string>("foo", "asdfs");
+    target->getBondWithIdx(0)->setProp<std::string>("bar", "dsafasdf");
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+
+    {
+      std::string pkl;
+      MolPickler::pickleMol(*mol, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 1);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol2, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol3, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    }
+  }
+  SECTION("basics EBV") {
+    auto mol = "CC"_smarts;
+    auto target = "CC"_smiles;
+    REQUIRE(mol);
+    REQUIRE(target);
+
+    ExplicitBitVect bv(10);
+    bv.setBit(0);
+    bv.setBit(2);
+    bv.setBit(3);
+    bv.setBit(5);
+    bv.setBit(7);
+    bv.setBit(9);
+
+    RWMol mol2(*mol);
+    RWMol mol3(*mol);
+
+    mol->getAtomWithIdx(0)->expandQuery(
+        makePropQuery<Atom, ExplicitBitVect>("foo", bv, 0.0));
+    mol->getBondWithIdx(0)->expandQuery(
+        makePropQuery<Bond, ExplicitBitVect>("bar", bv, 0.0));
+    ExplicitBitVect bv2(bv);
+    bv2.unsetBit(9);
+    mol2.getAtomWithIdx(0)->expandQuery(
+        makePropQuery<Atom, ExplicitBitVect>("foo", bv2, 0.0));
+    mol3.getBondWithIdx(0)->expandQuery(
+        makePropQuery<Bond, ExplicitBitVect>("bar", bv2, 0.0));
+
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+
+    target->getAtomWithIdx(0)->setProp<ExplicitBitVect>("foo", bv);
+    target->getBondWithIdx(0)->setProp<ExplicitBitVect>("bar", bv);
+    CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    {
+      std::string pkl;
+      MolPickler::pickleMol(*mol, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 1);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, *mol, ps).size() == 1);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol2, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol2, ps).size() == 0);
+    }
+    {
+      std::string pkl;
+      MolPickler::pickleMol(mol3, pkl);
+      RWMol pklmol(pkl);
+      REQUIRE(pklmol.getAtomWithIdx(0)->hasQuery());
+      REQUIRE(pklmol.getBondWithIdx(0)->hasQuery());
+      CHECK(SubstructMatch(*target, pklmol, ps).size() == 0);
+      // make sure we are idempotent in pickling
+      CHECK(SubstructMatch(*target, mol3, ps).size() == 0);
+    }
+  }
+}
+
+TEST_CASE("specified query matches unspecified atom") {
+  SECTION("atom basics") {
+    auto q = "F[C@](Cl)(Br)C"_smarts;
+    REQUIRE(q);
+
+    auto m1 = "F[C@](Cl)(Br)C"_smiles;
+    REQUIRE(m1);
+    auto m2 = "FC(Cl)(Br)C"_smiles;
+    REQUIRE(m2);
+    auto m3 = "F[C@@](Cl)(Br)C"_smiles;
+    REQUIRE(m3);
+
+    SubstructMatchParameters ps;
+    ps.useChirality = true;
+    CHECK(SubstructMatch(*m1, *q, ps).size() == 1);
+    CHECK(SubstructMatch(*m2, *q, ps).empty());
+    CHECK(SubstructMatch(*m3, *q, ps).empty());
+
+    ps.specifiedStereoQueryMatchesUnspecified = true;
+    CHECK(SubstructMatch(*m1, *q, ps).size() == 1);
+    CHECK(SubstructMatch(*m2, *q, ps).size() == 1);
+    CHECK(SubstructMatch(*m3, *q, ps).empty());
+  }
+  SECTION("bond basics") {
+    auto q = "F/C=C/Br"_smarts;
+    REQUIRE(q);
+
+    auto m1 = "F/C=C/Br"_smiles;
+    REQUIRE(m1);
+    auto m2 = "FC=CBr"_smiles;
+    REQUIRE(m2);
+    auto m3 = "F/C=C\\Br"_smiles;
+    REQUIRE(m3);
+
+    SubstructMatchParameters ps;
+    ps.useChirality = true;
+    CHECK(SubstructMatch(*m1, *q, ps).size() == 1);
+    CHECK(SubstructMatch(*m2, *q, ps).empty());
+    CHECK(SubstructMatch(*m3, *q, ps).empty());
+
+    ps.specifiedStereoQueryMatchesUnspecified = true;
+    std::cerr << "m1" << std::endl;
+    CHECK(SubstructMatch(*m1, *q, ps).size() == 1);
+    std::cerr << "m2" << std::endl;
+    CHECK(SubstructMatch(*m2, *q, ps).size() == 1);
+    std::cerr << "m3" << std::endl;
+    CHECK(SubstructMatch(*m3, *q, ps).empty());
+  }
 }
\ No newline at end of file
