@@ -21,7 +21,13 @@
 #include <RDBoost/PySequenceHolder.h>
 #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #include <numpy/arrayobject.h>
+
+#include <RDGeneral/BoostStartInclude.h>
+#include <boost/python/numpy.hpp>
+#include <RDGeneral/BoostEndInclude.h>
+
 namespace python = boost::python;
+namespace np = boost::python::numpy;
 
 namespace RDKit {
 RDGeom::Point3D GetAtomPos(const Conformer *conf, unsigned int aid) {
@@ -52,6 +58,46 @@ PyObject *GetPos(const Conformer *conf) {
   return PyArray_Return(res);
 }
 
+void SetPos(Conformer *conf, np::ndarray const &array) {
+  if (array.get_dtype() != np::dtype::get_builtin<double>()) {
+    PyErr_SetString(PyExc_TypeError, "Incorrect array data type");
+    python::throw_error_already_set();
+  }
+
+  if (array.get_nd() != 2) {
+    PyErr_SetString(PyExc_TypeError, "Input array shape must be of rank 2");
+    python::throw_error_already_set();
+  }
+
+  if (array.shape(0) != conf->getNumAtoms()) {
+    PyErr_SetString(
+        PyExc_ValueError,
+        "Position array shape doesn't equal the number of atoms in the conformer");
+    python::throw_error_already_set();
+  }
+
+  if (array.shape(1) < 2 || array.shape(1) > 3) {
+    PyErr_SetString(PyExc_ValueError,
+                    "Position array point dimension must be 2 or 3 (2d or 3d)");
+    python::throw_error_already_set();
+  }
+
+  const auto *data = reinterpret_cast<double *>(array.get_data());
+  RDGeom::POINT3D_VECT &pos = conf->getPositions();
+  if (array.shape(1) == 2) {
+    for (size_t i = 0; i < conf->getNumAtoms(); ++i) {
+      pos[i].x = data[i * 2];
+      pos[i].y = data[i * 2 + 1];
+      pos[i].z = 0.0;
+    }
+  } else {
+    for (size_t i = 0; i < conf->getNumAtoms(); ++i) {
+      pos[i].x = data[i * 3];
+      pos[i].y = data[i * 3 + 1];
+      pos[i].z = data[i * 3 + 2];
+    }
+  }
+}
 void SetAtomPos(Conformer *conf, unsigned int aid, python::object loc) {
   // const std::vector<double> &loc) {
   int dim = python::extract<int>(loc.attr("__len__")());
@@ -66,35 +112,48 @@ std::string confClassDoc =
 
 struct conformer_wrapper {
   static void wrap() {
-    python::class_<Conformer, CONFORMER_SPTR>("Conformer", confClassDoc.c_str(),
-                                              python::init<>())
+    python::class_<Conformer, CONFORMER_SPTR>(
+        "Conformer", confClassDoc.c_str(), python::init<>(python::args("self")))
         .def(python::init<unsigned int>(
+            python::args("self", "numAtoms"),
             "Constructor with the number of atoms specified"))
-        .def(python::init<const Conformer &>())
+        .def(python::init<const Conformer &>(python::args("self", "other")))
 
-        .def("GetNumAtoms", &Conformer::getNumAtoms,
+        .def("GetNumAtoms", &Conformer::getNumAtoms, python::args("self"),
              "Get the number of atoms in the conformer\n")
 
-        .def("HasOwningMol", &Conformer::hasOwningMol,
+        .def("HasOwningMol", &Conformer::hasOwningMol, python::args("self"),
              "Returns whether or not this instance belongs to a molecule.\n")
         .def("GetOwningMol", &Conformer::getOwningMol,
              "Get the owning molecule\n",
-             python::return_value_policy<python::reference_existing_object>())
+             python::return_value_policy<python::reference_existing_object>(),
+             python::args("self"))
 
-        .def("GetId", &Conformer::getId, "Get the ID of the conformer")
-        .def("SetId", &Conformer::setId, "Set the ID of the conformer\n")
+        .def("GetId", &Conformer::getId, python::args("self"),
+             "Get the ID of the conformer")
+        .def("SetId", &Conformer::setId, python::args("self", "id"),
+             "Set the ID of the conformer\n")
 
-        .def("GetAtomPosition", GetAtomPos, "Get the posistion of an atom\n")
-        .def("GetPositions", GetPos, "Get positions of all the atoms\n")
-        .def("SetAtomPosition", SetAtomPos,
+        .def("GetAtomPosition", GetAtomPos, python::args("self", "aid"),
+             "Get the posistion of an atom\n")
+        .def("GetPositions", GetPos, python::args("self"),
+             "Get positions of all the atoms\n")
+        .def(
+            "SetPositions", SetPos,
+            (python::args("self"), python::args("positions")),
+            "Set positions of all the atoms given a 2D or 3D numpy array of type double\n")
+        .def("SetAtomPosition", SetAtomPos, python::args("self", "aid", "loc"),
              "Set the position of the specified atom\n")
         .def("SetAtomPosition",
              (void(Conformer::*)(unsigned int, const RDGeom::Point3D &)) &
                  Conformer::setAtomPos,
+             python::args("self", "atomId", "position"),
              "Set the position of the specified atom\n")
 
-        .def("Set3D", &Conformer::set3D, "Set the 3D flag of the conformer\n")
-        .def("Is3D", &Conformer::is3D, "returns the 3D flag of the conformer\n")
+        .def("Set3D", &Conformer::set3D, python::args("self", "v"),
+             "Set the 3D flag of the conformer\n")
+        .def("Is3D", &Conformer::is3D, python::args("self"),
+             "returns the 3D flag of the conformer\n")
 
         // properties
         .def("SetProp", MolSetProp<Conformer, std::string>,
@@ -148,7 +207,7 @@ struct conformer_wrapper {
              "    - computed: (optional) marks the property as being "
              "computed.\n"
              "                Defaults to False.\n\n")
-        .def("HasProp", MolHasProp<Conformer>,
+        .def("HasProp", MolHasProp<Conformer>, python::args("self", "key"),
              "Queries a conformer to see if a particular property has been "
              "assigned.\n\n"
              "  ARGUMENTS:\n"
@@ -166,6 +225,7 @@ struct conformer_wrapper {
             "    - If the property has not been set, a KeyError exception "
             "will be raised.\n")
         .def("GetDoubleProp", GetProp<Conformer, double>,
+             python::args("self", "key"),
              "Returns the double value of the property if possible.\n\n"
              "  ARGUMENTS:\n"
              "    - key: the name of the property to return (a string).\n\n"
@@ -173,7 +233,7 @@ struct conformer_wrapper {
              "  NOTE:\n"
              "    - If the property has not been set, a KeyError exception "
              "will be raised.\n")
-        .def("GetIntProp", GetProp<Conformer, int>,
+        .def("GetIntProp", GetProp<Conformer, int>, python::args("self", "key"),
              "Returns the integer value of the property if possible.\n\n"
              "  ARGUMENTS:\n"
              "    - key: the name of the property to return (a string).\n\n"
@@ -182,6 +242,7 @@ struct conformer_wrapper {
              "    - If the property has not been set, a KeyError exception "
              "will be raised.\n")
         .def("GetUnsignedProp", GetProp<Conformer, unsigned int>,
+             python::args("self", "key"),
              "Returns the unsigned int value of the property if possible.\n\n"
              "  ARGUMENTS:\n"
              "    - key: the name of the property to return (a string).\n\n"
@@ -190,6 +251,7 @@ struct conformer_wrapper {
              "    - If the property has not been set, a KeyError exception "
              "will be raised.\n")
         .def("GetBoolProp", GetProp<Conformer, bool>,
+             python::args("self", "key"),
              "Returns the Bool value of the property if possible.\n\n"
              "  ARGUMENTS:\n"
              "    - key: the name of the property to return (a string).\n\n"
@@ -197,12 +259,13 @@ struct conformer_wrapper {
              "  NOTE:\n"
              "    - If the property has not been set, a KeyError exception "
              "will be raised.\n")
-        .def("ClearProp", MolClearProp<Conformer>,
+        .def("ClearProp", MolClearProp<Conformer>, python::args("self", "key"),
              "Removes a property from the conformer.\n\n"
              "  ARGUMENTS:\n"
              "    - key: the name of the property to clear (a string).\n")
 
         .def("ClearComputedProps", MolClearComputedProps<Conformer>,
+             python::args("self"),
              "Removes all computed properties from the conformer.\n\n")
         .def("GetPropNames", &Conformer::getPropList,
              (python::arg("self"), python::arg("includePrivate") = false,
