@@ -185,7 +185,10 @@ void testRDAny() {
       TEST_ASSERT(rdany_cast<std::vector<double>>(b)[i] == i);
     }
   }
-  const int loops = 10000000;
+
+  // growth in the loops below is loop * loops / 2, so going higher
+  // than this will cause an overflow of std::any_cast<int>(*v)
+  const int loops = sqrt(std::numeric_limits<int>::max());
   {
     std::clock_t clock1 = std::clock();
     std::any v;
@@ -424,7 +427,7 @@ void testVectToString() {
     d.setVal("foo", v);
     std::string sv;
     d.getVal("foo", sv);
-    TEST_ASSERT(sv == "[1,0,]");
+    TEST_ASSERT(sv == "[1,0]");
   }
   {
     Dict d;
@@ -434,9 +437,9 @@ void testVectToString() {
     d.setVal("foo", v);
     std::string sv;
     d.getVal("foo", sv);
-    TEST_ASSERT(sv == "[1,0,]");
+    TEST_ASSERT(sv == "[1,0]");
     sv = d.getVal<std::string>("foo");
-    TEST_ASSERT(sv == "[1,0,]");
+    TEST_ASSERT(sv == "[1,0]");
   }
   {
     Dict d;
@@ -446,9 +449,9 @@ void testVectToString() {
     d.setVal("foo", v);
     std::string sv;
     d.getVal("foo", sv);
-    TEST_ASSERT(sv == "[1.2,0,]");
+    TEST_ASSERT(sv == "[1.2,0]");
     sv = d.getVal<std::string>("foo");
-    TEST_ASSERT(sv == "[1.2,0,]");
+    TEST_ASSERT(sv == "[1.2,0]");
   }
   {
     Dict d;
@@ -458,9 +461,9 @@ void testVectToString() {
     d.setVal("foo", v);
     std::string sv;
     d.getVal("foo", sv);
-    TEST_ASSERT(sv == "[10001,0,]");
+    TEST_ASSERT(sv == "[10001,0]");
     sv = d.getVal<std::string>("foo");
-    TEST_ASSERT(sv == "[10001,0,]");
+    TEST_ASSERT(sv == "[10001,0]");
   }
 
   BOOST_LOG(rdErrorLog) << "\tdone" << std::endl;
