@@ -32,6 +32,7 @@ const std::string _BondsPotentialStereo = "_BondsPotentialStereo";
 const std::string _ChiralAtomRank = "_chiralAtomRank";
 const std::string _CIPCode = "_CIPCode";
 const std::string _CIPRank = "_CIPRank";
+const std::string _CanonicalRankingNumber = "_CanonicalRankingNumber";
 const std::string _ChiralityPossible = "_ChiralityPossible";
 const std::string _CrippenLogP = "_CrippenLogP";
 const std::string _CrippenMR = "_CrippenMR";
@@ -49,6 +50,7 @@ const std::string _MolFileBondCfg = "_MolFileBondCfg";
 
 const std::string _Name = "_Name";
 const std::string _NeedsQueryScan = "_NeedsQueryScan";
+const std::string _NonExplicit3DChirality = "_NonExplicit3DChirality";
 const std::string _QueryFormalCharge = "_QueryFormalCharge";
 const std::string _QueryHCount = "_QueryHCount";
 const std::string _QueryIsotope = "_QueryIsotope";
@@ -86,6 +88,8 @@ const std::string _queryRootAtom = "_queryRootAtom";
 const std::string _ringStereoAtoms = "_ringStereoAtoms";
 const std::string _ringStereoWarning = "_ringStereoWarning";
 const std::string _ringStereochemCand = "_ringStereochemCand";
+const std::string _ringStereoOtherAtom = "_ringStereoOtherAtom";
+const std::string _mesoOtherAtom = "_mesoOtherAtom";
 const std::string _chiralPermutation = "_chiralPermutation";
 const std::string _smilesAtomOutputOrder = "_smilesAtomOutputOrder";
 const std::string _smilesBondOutputOrder = "_smilesBondOutputOrder";
@@ -97,6 +101,8 @@ const std::string _unspecifiedOrder = "_unspecifiedOrder";
 const std::string _brokenChirality = "_brokenChirality";
 const std::string _rgroupAtomMaps = "_rgroupAtomMaps";
 const std::string _rgroupBonds = "_rgroupBonds";
+const std::string _rgroupTargetAtoms = "_rgroupTargetAtoms";
+const std::string _rgroupTargetBonds = "_rgroupTargetBonds";
 const std::string dummyLabel = "dummyLabel";
 const std::string extraRings = "extraRings";
 const std::string isImplicit = "isImplicit";
@@ -112,13 +118,12 @@ const std::string molRxnRole = "molRxnRole";
 const std::string molTotValence = "molTotValence";
 const std::string molFileLinkNodes = "_molLinkNodes";
 const std::string numArom = "numArom";
-const std::string origNoImplicit = "origNoImplicit";
 const std::string ringMembership = "ringMembership";
 const std::string smilesSymbol = "smilesSymbol";
 const std::string atomLabel = "atomLabel";
 const std::string OxidationNumber = "OxidationNumber";
 const std::string internalRgroupSmiles = "internalRgroupSmiles";
-
+const std::string molRingBondCount = "molRingBondCount";
 const std::string molSubstCount = "molSubstCount";
 const std::string molAttachPoint = "molAttchpt";
 const std::string molAttachOrder = "molAttchord";
@@ -126,6 +131,7 @@ const std::string molAtomClass = "molClass";
 const std::string molAtomSeqId = "molSeqid";
 const std::string molRxnExactChange = "molRxnExachg";
 const std::string molReactStatus = "molReactStatus";
+const std::string _fromAttachPoint = "_fromAttchpt";
 
 const std::string molNote = "molNote";
 const std::string atomNote = "atomNote";
